import numpy as np
import sys
import matplotlib.pyplot as plt
from obspy import read, Trace, Stream
from PyQt4 import QtGui
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
from basic_functions_classifier import *
from mydetails import info

class Window(QtGui.QDialog):
    """
    The GUI CLASS containing all the buttons and plotting functions etc

    """
    def __init__(self, mode='full', parent=None):
        global name1
        print('Starting up the GUI.....')
        self.qgood = 0
        self.qbad  = 0
        self.quality_l = []
        self.quality = []
        self.counter = 0
        self.velocity = 3800.  # Can be adjusted
        super(Window, self).__init__(parent)
        self.figure = plt.figure()
        self.canvas = FigureCanvas(self.figure)

        self.qualitylabel = QtGui.QLabel(self)
        self.qualitylabel.setText('Initial screen, Here will be displayed the quality!')
        
        if mode!='full': 
            self.qualitylabel.setText('Display only mode, already classified, quality:'+o.whichones)

        self.qualitylabel.setAlignment(Qt.AlignCenter)

        self.button_next = QtGui.QPushButton('next')
        self.button_prev = QtGui.QPushButton('previous')

        self.button_g = QtGui.QPushButton('good (arrow up)')
        self.button_b = QtGui.QPushButton('bad  (arrow down)')
        self.button_s = QtGui.QPushButton('save and quit (automatically after last trace)')
        
        if mode!='full': 
            self.button_g.setEnabled(False)
            self.button_b.setEnabled(False)
            self.button_s.setEnabled(False)

        if mode=='full': 
            self.button_next.setEnabled(False)
            self.button_prev.setEnabled(False)

        self.button_next.clicked.connect(self.next)
        self.button_prev.clicked.connect(self.prev)

        self.button_g.clicked.connect(self.good)
        self.button_b.clicked.connect(self.bad)
        self.button_s.clicked.connect(self.save)

        layout = QtGui.QVBoxLayout()
        hbox   = QtGui.QHBoxLayout()

        hbox.addWidget(self.button_b)
        hbox.addWidget(self.button_g)

        hbox0   = QtGui.QHBoxLayout()
        hbox0.addWidget(self.button_prev)
        hbox0.addWidget(self.button_next)
        
        layout.addWidget(self.qualitylabel)
        layout.addWidget(self.canvas)
        layout.addLayout(hbox0)
        layout.addLayout(hbox)
        layout.addWidget(self.button_s)
        self.setLayout(layout)

        self.setGeometry(1600, 1600, 1600, 500)
        self.setWindowTitle('Classifier')

    def keyPressEvent(self, eventQKeyEvent):
        key = eventQKeyEvent.key()
        if key == Qt.Key_Escape:
            print("pressed Escape button, closing down...")
            self.close()
        if key == Qt.Key_Up:
            self.good()
        elif key == Qt.Key_Down:
            self.bad()


    def next(self):
        self.counter += 1
        self.plot()

    def prev(self):
        self.counter -= 1
        self.plot()

    def plot(self):
        # if classi == 'synth':
        #     self.synthetics()

        data = o.stream_[self.counter].data
        ax = self.figure.add_subplot(111)
        ax.set_ylim([-1.2, 1.3])
        # ax.hold(False)
        plt.cla()
        ax.plot(data, label = str(self.counter+1)+'/'+str(len(o.corrlist))+'   '+o.corrlist[self.counter], 
                color='k', linewidth=2)
        # ax.hold(True)
        # if classi =='synth':
        #     ax.plot(self.synth, color='r', linewidth='2')
        try: 
            ax.axvline(o.stream_[self.counter].stats.sac.dist / self.velocity, color='r', linewidth='2')
        except: 
            pass
        # ax.plot(f.reconstructions[self.counter], color='blue')
        ax.legend(loc = 2, fontsize = "small", borderaxespad=0.)
        ax.grid(True)
        ax.set_ylim([-1.2, 1.3])
        self.canvas.draw()

    def good(self):
        try:
            self.qgood += 1
            self.change_label()
            self.q = 1
            self.classify()
            self.next()
            

        except IndexError:
            self.err1()
            # self.counter -= 1

    def err1(self):
        print("Indexing error, you might have reached the end")
        self.save()
        self.button_g.setEnabled(False)
        self.button_b.setEnabled(False)


    def bad(self):
        try: 
            self.qbad += 1
            self.change_label()
            self.q = 0
            self.classify()
            self.next()
        except IndexError:
            self.err1()

    def change_label(self):
        text = (('Good Quality: \t %3.0f / %.0f \t (%3.1f)%% \t \t Classified Traces: '+str(self.counter+1)+'/'+str(len(o.corrlist))) 
                % (self.qgood, self.counter+1, (float(self.qgood)/(self.counter+1))*100.))

        print(text)
        self.qualitylabel.setText(text)

    def classify(self):
        self.quality_l.append([o.corrlist[self.counter], self.q])

    def save(self): 
        global fname
        project = proj
        filename = path_classifications_hand + '/'+project+'/' + fname
        np.save(filename, self.quality_l)
        self.close()
        print("saved:", filename)


def classifications(mode = 'full', whichones = 'all', name='patrick', path_to_folder='/your_path/to/folder/',
                    project='your+project_name', fname='file_identifyer', path_waveforms='specify'):
    global o
    global f
    global path_classifications_hand
    # global path_classifications_synthetic
    global name1
    global proj
    proj = project
    name1 = name
    path_postprocessed, path_classifications_hand, path_subsets, path_waveforms = paths(type='*.SAC')

    o = ostream(path_waveforms)
    # o.corrlist = np.array(o.corrlist)[[x]].tolist()
    o.normalize(type='positive_max')


    app = QtGui.QApplication(sys.argv)
    main = Window(mode=mode)
    main.plot()
    print('GUI Ready and Running')
    main.show()
    # plt.figure()
    sys.exit(app.exec_())

def jupyter_run():
    global fname
    name, path, project, fname, path_waveforms = info()
    classifications(name=name, path_to_folder=path, project=project, fname=fname, path_waveforms=path_waveforms)


if __name__ == '__main__':
    jupyter_run()
