import numpy as np

import matplotlib
matplotlib.use('TkAgg')
import matplotlib.pyplot as plt
from obspy import read, Trace
from    scipy.integrate import simps
from scipy import interpolate


class Data:
    def __init__(self):
        self.stream = read('trace.SAC')
        self.data = np.load('data.npy')
        self.feat = np.load('features.npy')
        # print(self.feat)
        self.name = np.load('names.npy')
        self.reco = np.load('reconstructions2.npy')
        self.rec2 = np.load('reconstructions1.npy')
        self.rena = self.rec2.copy()
        self.rena[0][self.rena[0] == 0] = np.nan

    def plotter(self):
        f, axarr = plt.subplots(1, sharex='col', figsize=(18, 6))
        f.subplots_adjust(hspace=0.03)
        f.subplots_adjust(wspace=0.03)
        # f.hold(True)
        feats = self.feat[0][0]
        # print(feats)
        axarr.plot(self.data,
                   label='Correlation $\eta (t)$:  ' + 'self.name', c='dimgrey',
                   linewidth=2.)

        axarr.plot(self.rena[0],
                   label=('WBFE Waveform $W_0$'),
                   color='black',
                   linewidth=2.)
        # print feats[0][1]
        axarr.annotate('eta', xy=(feats[0] + feats[0] / 2., feats[-1] * feats[3]), xycoords='data',
                       xytext=(-50, 20), textcoords='offset points',
                       arrowprops=dict(arrowstyle='->'), size=10)

        axarr.annotate('t1', xy=(feats[0], 0), xycoords='data',
                       xytext=(-20, -50), textcoords='offset points',
                       arrowprops=dict(arrowstyle='->', lw=2),
                       size=10)

        plt.show()


class Data_mass():
    def __init__(self,i):
        self.stream = read('trace'+str(i)+'.SAC')
        self.data = np.load('data'+str(i)+'.npy')
        self.feat = np.load('features'+str(i)+'.npy')
        # print(self.feat)
        self.name = np.load('names'+str(i)+'.npy')
        self.reco = np.load('reconstructions'+str(i)+'2.npy')
        self.rec2 = np.load('reconstructions'+str(i)+'1.npy')
        self.rena = self.rec2.copy()
        self.rena[0][self.rena[0] == 0] = np.nan

    def plotter(self):
        f, axarr = plt.subplots(1, sharex='col', figsize=(18, 6))
        f.subplots_adjust(hspace=0.03)
        f.subplots_adjust(wspace=0.03)
        # f.hold(True)
        feats = self.feat[0][0]
        # print(feats)
        axarr.plot(self.data,
                   label='Correlation $\eta (t)$:  ' + 'self.name', c='dimgrey',
                   linewidth=2.)

        axarr.plot(self.rena[0],
                   label=('WBFE Waveform $W_0$'),
                   color='black',
                   linewidth=2.)
        # print feats[0][1]
        axarr.annotate('eta', xy=(feats[0] + feats[0] / 2., feats[-1] * feats[3]), xycoords='data',
                       xytext=(-50, 20), textcoords='offset points',
                       arrowprops=dict(arrowstyle='->'), size=10)

        axarr.annotate('t1', xy=(feats[0], 0), xycoords='data',
                       xytext=(-20, -50), textcoords='offset points',
                       arrowprops=dict(arrowstyle='->', lw=2),
                       size=10)

        plt.show()

def FT(data):
    #print("Running FT()")
    """
    Returns the FFT of data
    """
    return (np.abs(np.fft.fft(data))**2)[0:len(data)//2]

def FTx(data):
    #print "running FTx()"
    """
    Returns the frequencies of the FFT from the data
    """
    return np.fft.fftfreq(data.size, 1.)[0:len(data)//2]


def Resample(ft, periods, pmin=20, pmax=130, intervals=18, distance=0):
    """
    Resample the ft (from FT) with the('periods') (from FTx)
    periods as number of periods within the period bandwidth from
    pmin to
    pmax

    returns:
    nx:     The x-values, the periods for the resampled FFT-spectrum
    ny:     The resampled FFT-spectrum
    """
    # FT from FT
    intervals = intervals - 1
    # print "running Resample()"
    s = interpolate.interp1d(periods, ft)
    idx = np.all([periods >= pmin, periods <= pmax], axis=0)
    x = periods[idx]
    y = ft[idx]

    xnew = np.linspace(pmin, pmax, num=10000, endpoint=True)
    ints = np.linspace(pmin, pmax, num=intervals, endpoint=True)

    delta = (1. / pmin - 1. / pmax) / intervals
    alpha = simps(s(xnew), xnew)
    nx = []
    ny = []
    for i in range(len(ints) - 1):
        min = ints[i]
        max = ints[i + 1]
        # #print
        idx = np.all([xnew >= min, xnew <= max], axis=0)
        tmpx = xnew[idx]
        tmpy = s(tmpx)
        ttmpy = simps(tmpy, x=tmpx) / (alpha * delta)
        nx.append(tmpx[0] + (tmpx[-1] - tmpx[0]) / 2)
        ny.append(ttmpy)
    ny.append(alpha)
    ny.append(distance)

    # #print distance, alpha
    return nx, ny


def Resample(ft, periods, pmin=20, pmax=130, intervals=18, distance=0):
    """
    Resample the ft (from FT) with the('periods') (from FTx)
    periods as number of periods within the period bandwidth from
    pmin to
    pmax

    returns:
    nx:     The x-values, the periods for the resampled FFT-spectrum
    ny:     The resampled FFT-spectrum
    """
    # FT from FT
    intervals = intervals - 1
    # print "running Resample()"
    s = interpolate.interp1d(periods, ft)
    idx = np.all([periods >= pmin, periods <= pmax], axis=0)
    x = periods[idx]
    y = ft[idx]

    xnew = np.linspace(pmin, pmax, num=10000, endpoint=True)
    ints = np.linspace(pmin, pmax, num=intervals, endpoint=True)

    delta = (1. / pmin - 1. / pmax) / intervals
    alpha = simps(s(xnew), xnew)
    nx = []
    ny = []
    for i in range(len(ints) - 1):
        min = ints[i]
        max = ints[i + 1]
        # #print
        idx = np.all([xnew >= min, xnew <= max], axis=0)
        tmpx = xnew[idx]
        tmpy = s(tmpx)
        ttmpy = simps(tmpy, x=tmpx) / (alpha * delta)
        nx.append(tmpx[0] + (tmpx[-1] - tmpx[0]) / 2)
        ny.append(ttmpy)
    ny.append(alpha)
    ny.append(distance)

    # #print distance, alpha
    return nx, ny


def resampler(data, pmin=30, pmax=130, intervals=18, return_orig=False):
    """
    Caller for the Resampler of the DATA within the period band from pmin to pmax and the number of intervals.

    If return_orig is True, then also the original (not resampled) FFT with the periods are returned.
    """
    distance = data.stats.sac.dist
    data = data.data

    # print 'running resampler()'
    ft = FT(data)[1:]
    ft = ft / np.max(ft)
    periods = 1. / FTx(data)[1:]
    # returns periods_resampled and FT_resampled
    if return_orig == True:
        x, y = Resample(ft, periods, pmin=pmin, pmax=pmax, intervals=intervals, distance=distance)
        return periods, ft, x, y
    else:
        return Resample(ft, periods, pmin=pmin, pmax=pmax, intervals=intervals, distance=distance)


def do_plot():
    mass = False
    print('Creating the plot...')
    if mass == False:
        D = Data()

        ft_feat_l  = D.feat[0][0:-2]
        ft_feat_l /= np.max(np.abs(ft_feat_l))

        trace       = D.stream[0]
        print(trace)
        # trace /= np.max(np.abs(trace))

        t_full, ft_full, t_feat, ft_feat = \
            resampler(trace, pmin=50, pmax=110, intervals=18, return_orig=True)

        ft_full /= np.max(np.abs(ft_full))

        ft_feat = ft_feat[0:-2]
        ft_feat /= np.max(np.abs(ft_feat))
        w = np.where((t_full < 150) & (t_full > 20))

        ft_full = ft_full[w]
        t_full = t_full[w]

        fig = plt.figure(figsize=(11,5))
        ax0 = plt.subplot(211)
        ax0.plot(trace.data,
                 color='black',
                 linewidth=1.5
                 )
        # plt.grid()
        ax0.set_ylabel('Norm. Ampl.')
        ax0.set_xlabel('Lag Time [s]')
        ax0.set_yticks([-1,0,1])
        ax0.set_xticks([0,1000,2000,2700])
        ax0.set_xlim([0,2700])

        ax1 = plt.subplot(212)
        ax1.plot(t_feat, ft_feat_l,
                 label=('FDFE features'),
                 color='grey',
                 linewidth=2,
                 marker='o',
                 linestyle='--'
                 # facecolor=None
                 )
        # ax1.plot(t_feat, ft_feat,
        #          label=('FDFE features calculated'),
        #          color='grey',
        #          linewidth=2.,
        #          marker='o'
        #          )
        ax1.plot(t_full, ft_full,
                 label=('FFT Full'),
                 color='black',
                 linewidth=1.5,
                 # facecolor=None
                 # marker='o'
                 )

        fig.patch.set_facecolor('none')
        ax1.set_ylabel('Norm. PS', size=10)
        ax1.set_xlabel('Period [s]', size=10)

        # ax1.xaxis.set_major_formatter(plt.NullFormatter())
        # ax1.yaxis.set_major_formatter(plt.NullFormatter())
        ax1.set_yticks([-0, 0.5, 1])
        ax1.set_xticks([30, 50, 75, 100, 125, 150])
        ax1.set_xlim([30,125])
        ax1.tick_params(axis='x', labelsize=10)
        ax1.tick_params(axis='y', labelsize=10)

        for item in ([ax1.title, ax1.xaxis.label, ax1.yaxis.label] +
                         ax1.get_xticklabels() + ax1.get_yticklabels()):
            item.set_fontsize(14)
        for item in ([ax0.title, ax0.xaxis.label, ax0.yaxis.label] +
                         ax0.get_xticklabels() + ax0.get_yticklabels()):
            item.set_fontsize(14)
        plt.grid()
        plt.tight_layout()
        plt.show()

    else:
        for j in [14, 15, 29, 22]:
            D = Data_mass(j)

            ft_feat_l = D.feat[0][0:-2]
            ft_feat_l /= np.max(np.abs(ft_feat_l))

            trace = D.stream[0]
            print(trace)
            # trace /= np.max(np.abs(trace))

            t_full, ft_full, t_feat, ft_feat = \
                resampler(trace, pmin=50, pmax=110, intervals=18, return_orig=True)

            ft_full /= np.max(np.abs(ft_full))

            ft_feat = ft_feat[0:-2]
            ft_feat /= np.max(np.abs(ft_feat))
            w = np.where((t_full < 150) & (t_full > 20))

            ft_full = ft_full[w]
            t_full = t_full[w]

            fig = plt.figure(figsize=(11, 8))
            ax0 = plt.subplot(211)
            ax0.plot(trace.data,
                     color='black',
                     linewidth=1.5,
                     label='Processed Noise Correlation'
                     )
            # plt.grid()
            ax0.set_ylabel('Norm. Amplitude')
            ax0.set_xlabel('Lag Time [s]')
            ax0.set_yticks([-1, 0, 1])
            ax0.set_xticks([0, 1000, 2000, 2700])
            ax0.set_xlim([0, 2700])
            if j == 14:
                plt.legend(loc=1, prop={'size':16})
            ax1 = plt.subplot(212)
            ax1.plot(t_feat, ft_feat_l,
                     label=('FDFE Features Noise Correlation'),
                     color='grey',
                     linewidth=2,
                     marker='o',
                     linestyle='--'
                     # facecolor=None
                     )
            # ax1.plot(t_feat, ft_feat,
            #          label=('FDFE features calculated'),
            #          color='grey',
            #          linewidth=2.,
            #          marker='o'
            #          )
            ax1.plot(t_full, ft_full,
                     label=('FFT Noise Correlation'),
                     color='black',
                     linewidth=1.5,
                     # facecolor=None
                     # marker='o'
                     )
            if j == 14:
                plt.legend(loc=2, prop={'size':16})
            fig.patch.set_facecolor('none')
            ax1.set_ylabel('Norm. Power Spectrum', size=10)
            ax1.set_xlabel('Period [s]', size=10)

            # ax1.xaxis.set_major_formatter(plt.NullFormatter())
            # ax1.yaxis.set_major_formatter(plt.NullFormatter())
            ax1.set_yticks([-0, 0.5, 1])
            ax1.set_xticks([30, 50, 75, 100, 125, 150])
            ax1.set_xlim([30, 125])
            ax1.tick_params(axis='x', labelsize=10)
            ax1.tick_params(axis='y', labelsize=10)
            for item in ([ax1.title, ax1.xaxis.label, ax1.yaxis.label] +
                             ax1.get_xticklabels() + ax1.get_yticklabels()):
                item.set_fontsize(16)
            for item in ([ax0.title, ax0.xaxis.label, ax0.yaxis.label] +
                             ax0.get_xticklabels() + ax0.get_yticklabels()):
                item.set_fontsize(16)

            plt.grid()
            plt.tight_layout()
            # plt.show()

            fig.savefig(str(j)+'.png')

if __name__ == '__main__':
    do_plot()