import numpy as np
import sys
import os
import csv
import control.NN_class as NN
import inspect
from mydetails import info
from pathlib import Path

from control.transfer_functions import *
from control.testing_functions import *

sys.path.append(os.path.abspath('..')+'/PREP/')
from control.NN_preparation import *
from control.parameters import defaults
# import click 

"""
Caller for the Neural Networks provided in the vv_vxx_class_.py file. 
The Dataset is a dummy dataset with two harmonics and noise, transformed into the frequency domain 
and resampled into 15 discrete frequencies to check allover performance.

The output will be saved into a file called: 
    "Filename.npz"


As reasonable was found as default values:
------------------------------------------------------------------
  samplenumber = 500       #Number of Samples within a dataset
  Filename     = "nn_v09_002_"+str(samplenumber)

  parameters   =((15,0,0,0),
                (60, sigmoid, sigmoid_p),
                (10, sigmoid, sigmoid_p),
                (2, linear, linear_p))

  bigiter     = 1000       #How often will a new dataset be created and used for training
  n_iter      = 201        #How often will a certain training dataset be passed (the order will change within every iteration!)
  max_err     = 1e-3        #Error limit
----------------------------------------------------------------------

"""
def writeit_(project_test ='/a20/' ,project_train= '/a20/', parameters ='default', file='meta', x='x', maxerr = 1e-6, niter = 100, numpyname = "/home/patrick/1Thesis/ann/Mai17/networks/network", classifile = 'a11_classified_hand_Laura', qcrit = 0.8, repeat=1, rank='1', convergence=False, mode='extr'):
  """
  Writes Results into the file with the variable name file
  """
  # mode = 'all'
  for ii in range(repeat):
    if mode in ['extr', 'all']:
        with open(file, 'a') as f_handle:
            i, bpnem, s, eg, eb, et, ngood, nbad, fname21        = networkfunction(project_train=project_train,project_test=project_test, parameters=parameters, ntype='extra', niter = niter, max_error = maxerr, numpyname = numpyname, classifile = classifile, qcrit = qcrit, i=ii, rank=rank, convergence=convergence)
            writer = csv.writer(f_handle, delimiter='\t', dialect='excel')
            rwstr = str("%12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s" % (
                str(x), str(classifile.split('_')[-1]), str("%1.3e" % qcrit), str("%1.3e" % maxerr), "WBFE",
                str(project_train),
                str(project_test), str('%10d' % niter), str(i), str('%.3e' % bpnem), str(s), str(ngood), str(nbad),
                str('%.2f' % eg), str('%.2f' % eb), str('%.2f' % et)))
            writer.writerow([rwstr])

    if mode in ['ft', 'all']:
        fi,fbpnem, fs, feg, feb, fet, fngood, fnbad, fname22 = networkfunction(project_train=project_train,project_test=project_test, parameters=parameters, ntype='ft', niter = niter, max_error = maxerr, numpyname = numpyname, classifile = classifile, qcrit = qcrit, i=ii, rank=rank,convergence=convergence)
        with open(file, 'a') as f_handle:
            writer = csv.writer(f_handle, delimiter='\t', dialect='excel')
            rwstr = str("%12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s" % (
                str(x), str(classifile.split('_')[-1]), str("%1.3e" % qcrit), str("%1.3e" % maxerr), "FDFE",
                str(project_train),
                str(project_test), str('%10d' % niter), str(i), str('%.3e' % fbpnem), str(s), str(fngood), str(fnbad),
                str('%.2f' % feg), str('%.2f' % feb), str('%.2f' % fet)))
            writer.writerow([rwstr])

    print("+--------start-TESTING--------------------+\n...")
    # print('\nTesting....')
    if qcrit == 1.3:
      tmptmp = [1.3, 1.2, 0.8, 0.9, 1.0, 1.1, 0.7, 0.6]
    else:
      tmptmp =[qcrit]
    for qcritt in tmptmp:

        if mode in ['extr', 'all']:
          ngood, nbad, eg, eb, et = testing(Filedir=fname21, ntype='extr', project_test=project_test, qcrit = qcritt, classifile=classifile, rank=rank)
          with open(file, 'a') as f_handle:
              writer = csv.writer(f_handle, delimiter = '\t', dialect = 'excel')
              # writer.writerow([str(x),classifile.split('_')[-1],str(qcritt),str(maxerr),'Textr',project_train,'/test/',str(niter),str(i), str('%.5f' % bpnem),str(s), str(ngood), str(nbad),str('%.3f' % eg),str('%.3f' % eb),str('%.3f' % et)])
              rwstr = str("%12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s" % (
              str(x), str(classifile.split('_')[-1]), str("%1.3e" % qcritt), str("%1.3e" % maxerr), "WBFE",
              str(project_train), str(project_test), str('%10d' % niter), str(i), str('%.3e' % bpnem), str(s), str(ngood),
              str(nbad), str('%.2f' % eg), str('%.2f' % eb), str('%.2f' % et)))
              writer.writerow([rwstr])

        if mode in ['ft', 'all']:
          ngood, nbad, eg, eb, et = testing(Filedir=fname21, ntype='ft', project_test=project_test, qcrit = qcritt, classifile=classifile, rank=rank)
          with open(file, 'a') as f_handle:
              writer = csv.writer(f_handle, delimiter='\t', dialect='excel')
              rwstr = str("%12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s" % (
                  str(x), str(classifile.split('_')[-1]), str("%1.3e" % qcritt), str("%1.3e" % maxerr), "FDFE",
                  str(project_train), str(project_test), str('%10d' % niter), str(i), str('%.3e' % bpnem), str(s),
                  str(ngood),
                  str(nbad), str('%.2f' % eg), str('%.2f' % eb), str('%.2f' % et)))
              writer.writerow([rwstr])

  if qcrit == 1.3:
      tmptmp = [1.3, 1.2, 0.8, 0.9, 1.0, 1.1, 0.7, 0.6]
  else:
      tmptmp = [qcrit]
  for qcritt in tmptmp:

    if mode in ['extr', 'all']:
      ngood, nbad, eg, eb, et = testing(Filedir=numpyname + project_train + project_test + '*extra*', ntype='extr',
                                        project_test=project_test, qcrit=qcritt, classifile=classifile, rank=rank)

    if mode in ['ft', 'all']:
            fngood, fnbad, feg, feb, fet = testing(Filedir=numpyname + project_train + project_test + '*ft*', ntype='ft',
                                        project_test=project_test, qcrit=qcritt, classifile=classifile, rank=rank)

    if mode in ['extr', 'all']:
      with open(file, 'a') as f_handle:
          writer = csv.writer(f_handle, delimiter='\t', dialect='excel')
          # writer.writerow([str(x),classifile.split('_')[-1],str(qcritt),str(maxerr),'Textr',project_train,'/test/',str(niter),str(i), str('%.5f' % bpnem),str(s), str(ngood), str(nbad),str('%.3f' % eg),str('%.3f' % eb),str('%.3f' % et)])
          rwstr = str("%12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s" % (
              str(x), str(classifile.split('_')[-1]), str("%1.3e" % qcritt), str("%1.3e" % maxerr), "SUM_WBFE"+str(ii+1),
              str(project_train), str(project_test), str('%10d' % niter), str(i), str('%.3e' % bpnem), str(s),
              str(ngood),
              str(nbad), str('%.2f' % eg), str('%.2f' % eb), str('%.2f' % et)))
          writer.writerow([rwstr])

    if mode in ['ft', 'all']:
        with open(file, 'a') as f_handle:
          writer = csv.writer(f_handle, delimiter='\t', dialect='excel')
          rwstr2 = str("%12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s" % (
              str(x), str(classifile.split('_')[-1]), str("%1.3e" % qcritt), str("%1.3e" % maxerr),
              "SUM_FDFE" + str(ii + 1),
              str(project_train), str(project_test), str('%10d' % niter), str(i), str('%.3e' % bpnem), str(s),
              str(fngood),
              str(fnbad), str('%.2f' % feg), str('%.2f' % feb), str('%.2f' % fet)))
          writer.writerow([rwstr2])

          # writer = csv.writer(f_handle, delimiter='\t', dialect='excel')
          # writer.writerow(
          #     [str(x), classifile.split('_')[-1], str(qcritt), str(maxerr), 'Textr' + str(ii + 1), project_train,
          #      '/a11b/', str(niter), str(i), str('%.4f' % bpnem), str(s), str(ngood), str(nbad), str('%.3f' % eg),
          #      str('%.3f' % eb), str('%.3f' % et)])
          # writer.writerow([str(x),classifile.split('_')[-1],str(qcrit),str(maxerr),'four',project_train,project_test,str(niter), str(fi),str(fs), str(fngood), str(fnbad), str('%.3f' % feg),str('%.3f' % feb),str('%.3f' % fet)])


  print('The testing results can be viewed in the file:\n', file)
  print("+---------end-TESTING--------------------+\n")
  return fname21


def networkfunction(project_train='/a11/',project_test='/a11/', parameters =((18,0,0,0),
                (80, sigmoid, sigmoid_p),
                (40, sigmoid, sigmoid_p),
                (20, sigmoid, sigmoid_p),
                # (10, sigmoid, sigmoid_p),
                (2, linear, linear_p)), type='extracter', filename='asdf.txt', ntype='extra', niter = 100, max_error = 1e-6, numpyname = "/home/patrick/1Thesis/ann/Mai17/networks/network", classifile = 'a11_classified_hand_Laura', qcrit = 0.8, i=1, rank='0', convergence=False):
    """
    Call the Neural Network Backpropagation with the parameters specified in parameters
    """
    np.random.seed()
    #-----------------------------------------
    """Please specify parameters:"""
    samplenumber = 0       #Number of Samples within a dataset

    bigiter     = 1     #How often will a new dataset be created and used for training
    n_iter      = niter     #How often will a certain training dataset be passed (the order will change within every iteration!)
    max_err     = max_error  #Error limit

    immediate_testing = True
    #-----------------------------------------
    """Initialize the Network:"""
    otherft = False
    if otherft == True:
      if ntype =='extra':
        parameters = parameters

      else:
        ttmp = list(parameters)
        ttmp1 = np.zeros(len(ttmp[0])).astype(int)
        ttmp1[0] = int(30)
        ttmp1 = tuple(ttmp1)
        ttmp[0] = ttmp1
        ttmp = tuple(ttmp)
        parameters = ttmp

    BPN = NN.Backpropagation_NN(parameters)
    BPN.rank = rank
    BPN.projj = project_train.split('/')[1]
    #-----------------------------------------
    """Training"""
    # print ("Training with %0.0f samples, recreating the samples every %0.0f iterations \nfor maximum of %0.0f repetitions and a max. allowed mean error of %0.2e \n...") % (samplenumber, n_iter, bigiter, max_err)
    for ii in range(bigiter):
      # print "_____________________________________________________________________________________________"
      # print "outer_Iteration ", ii

      if ntype=='extra':
        s,q = prep_network(project=project_train, classifile = classifile, rank=rank)
      elif ntype =='ft':
        s, q = prep_FT_network(project=project_train, classifile = classifile, rank=rank)
      samplenumber = len(s)
      BPN.ntypes = ntype

      learning_rate_hid, learning_rate_out, tau_hid, tau_out, eps = defaults()
      BPN.train_network(s,q, n_iter, max_err=max_err,
                        print_intermediate = True,
                        learning_rate_hid=learning_rate_hid,
                        learning_rate_out=learning_rate_out,
                        tau_hid= tau_hid,
                        tau_out= tau_out,
                        eps=eps,
                        convergence=convergence)

      if BPN.err_mean <= max_err:
        print("___________________________________________________________________________________________")
        print("Deviation limit reached, breaking the loop, preparing the network...")
        break
        print("Training successfully, last Error:",BPN.tmpe1,"after",BPN.tmpe2, "Iterations \n")
      else:
        pass

    Filename     = numpyname+project_train+project_test+'_'+ntype+str(i)+'rank'+rank+project_train.split('/')[1]+project_test.split('/')[1]
    # -----------------------------------------
    print("Saving trained Network to File:\n", Filename)
    print("+---------end-TRAINING--------------------+\n")
    # print("The Network can be run via the nn_vxx_calltrained_.py function by using the Filename:\n", Filename)
    #-----------------------------------------

    """Saving"""
    BPN.save_network(Filename)


    """Testing"""
    if immediate_testing ==True:

      if ntype=='extra':
        s,q = call_network(project=project_test, classifile = classifile, rank=rank)
      elif ntype=='ft':
        s,q = call_FT_network(project=project_test, classifile = classifile, rank=rank)
      out = BPN.predict(s)

      q_out = NN.out_quality(out, qcrit)
      q_q   = NN.out_quality(q, qcrit)

      good_index = np.where(q_q==1)
      bad_index  = np.where(q_q==0)
      ngood = len(good_index[0])
      nbad  = len(bad_index[0])

      gerror = np.sum(np.abs(q_out[good_index]-q_q[good_index]))/ngood*100
      berror = np.sum(np.abs(q_out[bad_index]-q_q[bad_index]))/nbad*100
      t_error = np.sum(np.abs(q_out-q_q))/len(q_out)*100

      # print ("Testing: \tFor %0.0f trainsamples (%0.0ftest), the classification error is %0.2f %%' (%.8s) \t(%.0f iter)")% (samplenumber, len(q_out),t_error, ntype, n_iter)
      # print("Testing: \tFor",samplenumber," trainsamples , the classification error is",t_error,"  \t(",n_iter,"iter)")
      # print(samplenumber, len(q_out),t_error, ntype, n_iter)

    else:
      print("No Testing performed after training")

    return BPN.repeat, BPN.err_mean, samplenumber, gerror, berror, t_error, ngood, nbad, Filename+'.npz'

def do_testing(projectlist = ['/a11/', '/a20/'],
               numpyname = "/home/patrick/1Thesis/ann/Mai17/networks/network",
               file = '/home/patrick/1Thesis/wavemanipulation/2classification/testing/meta',
               input_size = 18,
               setsize = range(100,101,1),
               hidlay1 = range(20,21),
               hidlay2 = range(20,21),
               hidlay3 = range(20,21),
               repeat  = 3,
               test_repeat = 2,
               maxerr = 1e-6, niter = 1000,
               classificationfile = 'a11_classified_hand_Laura.npy',
               classifile = 'a11_classified_hand_Laura',
               qcrit = 0.8,
               mult = True,
               rank='0',
               convergence=False,
               mode='extr'):
  """
  Perform a full testing operation with looping through different parameters
  """
  # print(numpyname, file, classifile)
  meta =  file+'_rank'+str(rank)+ '.csv'
  if os.path.exists(meta):
    pass
  else:
    with open(meta, 'a') as f_handle:
      writer = csv.writer(f_handle, delimiter = '\t', dialect = 'excel')
      twstring = str("%12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s %12s"
                     %('nnconfig','fname','qcr','maxerr','type','train','test','iter_t','iter', 'e_tr','sample','ngood', 'nbad', 'err(%)g', 'err(%)b', 'err(%)t'))
      writer.writerow([twstring])
  # """THE BIG LOOP FOR TESTING """
  # "SIZE OF THE DATASET"
  for n in setsize:

    # "FIRST HIDDEN LAYER"
    for a in hidlay1:

      # "SECOND HIDDEN LAYER"
      tmp = hidlay2
      if tmp == None:
        tmp = range(1)
      else:
        tmp = hidlay2
      for b in tmp:

        # "THIRD HIDDEN LAYER"
        tmp = hidlay3
        if tmp == None:
            tmp = range(1)
        else:
          tmp = hidlay3
        for c in tmp:

          # "Repeat how often?"
          for ii in range(1):
            x = []
            x.append(a)


            if hidlay2 == None:
               parameters =((input_size,0,0,0),
                              (a, sigmoid, sigmoid_p),
                              # (b, sigmoid, sigmoid_p),
                              # (c, sigmoid, sigmoid_p),
                              (2, linear, linear_p))
            else:
              x.append(b)
              # pass

              if hidlay3 == None:
                  parameters =((input_size,0,0,0),
                                (a, sigmoid, sigmoid_p),
                                (b, sigmoid, sigmoid_p),
                                # (c, sigmoid, sigmoid_p),
                                (2, linear, linear_p))
              else:
                  parameters = ((input_size,0,0,0),
                                (a, sigmoid, sigmoid_p),
                                (b, sigmoid, sigmoid_p),
                                (c, sigmoid, sigmoid_p),
                                (2, linear, linear_p))
                  x.append(c)

###############################################################################################################################################################################
            project_test = projectlist[1]
            project_train= projectlist[0]
            fname11 = writeit_(project_test =project_test ,project_train= project_train, parameters =parameters, file=meta, x=x, maxerr = maxerr, niter = niter, numpyname = numpyname, classifile = classifile, qcrit = qcrit, repeat = repeat, rank=rank, convergence=convergence, mode=mode)

def perform_testing(
                  rank,
                  n,
                  filelist,
                  qlist,
                  projectlists,
                  subsets,
                  nrange,
                  maxerrs,
                  options = None,
                  errorfile='/home/patrick/1Thesis/wavemanipulation/2classification/testing/error',
                  numpyname = "/home/patrick/1Thesis/nFINAL/FILES/networks/",
                  hidlay1 = range(15,16,30),
                  hidlay2 = range(70,71,20),
                  hidlay3 = None,
                  repeat = 1,
                  name_add = '',
                  convergence=False,
                  mode = 'extr'
                  ):
    """
    Caller/Wrapper function for the do_testing function
    """
    print('___________________________________________________________________')
    print('Running the script for the following configuration:\n')
    print('Output keys\t', rank)
    print('subset size\t', n)
    print('Classifile \t', filelist)
    print('Qualitycri \t', qlist)
    print('Train Test \t', projectlists)
    print('Create subs\t', subsets)
    print('iterations \t', nrange)
    print('maxerrors  \t', maxerrs)
    print('addoptions \t', options)
    # print('Error file \t', errorfile)
    print('___________________________________________________________________\n')
    path = os.path.dirname(os.path.abspath(inspect.stack()[0][1]))
    basepath = Path(path).parents[2]
    program_path = str(basepath)
    if options == None:
        for cfile in filelist:
          if subsets == True:
            print('CREATING SUBSET for', projectlists[0][0])
            subset_creator(n=n[0], project =projectlists[0][0], classificationfile = cfile, number2 = 2)
            if projectlists[0][0] != projectlists[0][1]:
              print('CREATING SUBSET for', projectlists[0][1])
              subset_creator(n=n[0], project =projectlists[0][1], classificationfile = cfile, number2 = 2)
          for i in n:
            for qq in qlist:
              for niter in nrange:
                for maxerr in maxerrs:
                    cfilesplit = cfile.split('.npy')[0]
                    for prolist in projectlists:
                      do_testing(projectlist = prolist,
                                 numpyname = numpyname,
                                 file = program_path+'/RESULTS/'+prolist[0].split('/')[1]+'__'+name_add,
                                 input_size = 18,
                                 setsize = range(90,91,1),
                                 hidlay1 = hidlay1,
                                 hidlay2 = hidlay2,
                                 hidlay3 = hidlay3,
                                 repeat  = repeat,
                                 test_repeat = 1,
                                 maxerr = maxerr, niter =niter,
                                 classificationfile = cfile,
                                 classifile = cfilesplit,
                                 qcrit = qq,
                                 mult=False,
                                 rank=rank,
                                 convergence=convergence,
                                 mode=mode)

if __name__ == "__main__":
  print("Please call via a file called RANK_PROJtrainPROJtest_.py (e.g. 0_aa_.py)")
