import numpy as np
import os, csv
from control.transfer_functions import*

class Backpropagation_NN(object): 
  def __init__(self, 
              network_parameters): 

    """ 
    Backpropagation Artificial Neural Network for CLASSIFICATION of whichever input provided.
    Version: 0.9, March 22 2016 (19:06)
    ---In Testing---
    """
#---------------------------------------------------------------------------------------------------------
    """
    The Parameters will have the form of: 
    (
      (number_of_input_nodes,0,0....,0 {remark: number of other layers with zeros: hidden and output}), ...
      (hidden_layer_1_number_of_nodes, function_hidden_layer_1, function_prime_hidden_layer_1), ...
      ...
      (hidden_layer_n_number_of_nodes, function_hidden_layer_n, function_prime_hidden_layer_n), ...
      (number_of_output_nodes, function_output_layer, function_prime_output_layer)
      )
    example for 15 inputs, 2 hidden layers with 20 nodes each and 2 output nodes:
    
    network_parameters=((15,0,0,0),
                        (20, logistic, logistic_prime),
                        (20, logistic, logistic_prime), 
                        (2, logistic, logistic_prime))
    """

    # """Inspired by: 
    # - A youtube tutorial on NNs in Python as a brief introduction: 
    #     https://www.youtube.com/watch?v=XqRUHEeiyCs&list=PLRyu4ecIE9tibdzuhJr94uQeKnOFkkbq6

    # - Some tutorials for NNs, also with backpropagation. The data input skelleton is from there:
    #     https://triangleinequality.wordpress.com/category/machine-learning/
    
    # - Paper by Valentine for improvement
    # """

    self.n_layers       = len(network_parameters)
    self.sizes          = [layer[0] for layer in network_parameters]
    self.functions      = [layer[1] for layer in network_parameters]
    self.primefunctions = [layer[2] for layer in network_parameters]
    self.Parameters     = network_parameters
    self.rank = ''
    self.projj = ''
    self.setup_network()
    # print("Successfully initialized with shape:", self.sizes, "\n")

  def setup_network(self): 
    self.initialize_matrizes()

  def initialize_matrizes(self):
    self.weights   = []
    self.biases   = []
    self.inputs    = []
    self.outputs   = []
    self.errors   = []
    # factor = 1./len(self.X)*10 #Test to scale the weights, but pretty unsuccessful
    factor = 1.
    for layer in range(self.n_layers-1): 
      np.random.seed()
      N = self.sizes[layer]
      M = self.sizes[layer+1]
      self.weights.append(np.random.uniform(-1.,1.,(M,N))*(1./18.))
      # self.weights.append(np.random.random_sample((M,N)))
      # self.biases.append(np.random.normal(0,1,(M,1)))
      self.biases.append(1.)
      # self.biases.append(1)
      self.inputs.append(np.zeros((N,1)))
      self.outputs.append(np.zeros((N,1)))
      self.errors.append(np.zeros((N,1)))
    
    N = self.sizes[-1]
    self.inputs.append(np.zeros((N,1)))
    self.outputs.append(np.zeros((N,1)))
    self.errors.append(np.zeros((N,1)))


  """Forward Propagation"""
  def feedforward_loop(self): 

    #----------------------------------------------------------------------------
    for i in range(1, self.n_layers): 
      self.inputs[i] =self.weights[i-1].dot(self.outputs[i-1])+self.biases[i-1]
      self.outputs[i]=self.functions[i](self.inputs[i])
    #----------------------------------------------------------------------------

  def forwardpropagation(self, input): 
    k = len(input)
    input.shape      = (k,1)
    self.inputs[0]   = input
    self.outputs[0]  = input
    self.feedforward_loop()

    return self.outputs[-1]

  "Training"

  def train_network(self,input, output_des, n_iter, max_err=1e-3,
                    print_intermediate = True,
                    learning_rate_hid=0.5, learning_rate_out=0.2, 
                    tau_hid= 2000., tau_out= 500., 
                    eps=0.05, convergence=False):
    self.X = input
    self.y = output_des
    self.learning_rate_hid  =  learning_rate_hid
    self.learning_rate_out  =  learning_rate_out
    self.eps = eps
    self.tau_h = tau_hid
    self.tau_o = tau_out

    self.train_loop(n_iter, print_intermediate=print_intermediate, max_err=max_err, convergence=convergence)

  def train_loop(self, n_iter, max_err=1e-4, print_intermediate=True, 
                adjust_learning_rate = True, adjust_probability=True, 
                quality_crit = 1.2, 
                P_start=0.5, q_start=1., writefile=False, convergence=False):
    
    # print("\nTraining...")
    print("+--------start-TRAINING--------------------+")
    self.qcrit = quality_crit
    self.Pcorr = adjust_probability
    self.adjuster = adjust_learning_rate
    
    N = self.X.shape[0]
    self.P = P_start
    #----------------------------------------------------------------------------
    #----------------------------------------------------------------------------
    for repeat in range(n_iter):
      self.q = q_start
      self.i = 1

      index = list(range(N))
      np.random.seed()
      np.random.shuffle(index)
      #----------------------------------------------------------------------------
      for row in index: 
        x = self.X[row]
        y = self.y[row]
        self.backpropagation(x,y)
        self.i += 1
      #----------------------------------------------------------------------------
      tmp0 = (self.predict(self.X)-self.y)**2
      # print(tmp0)
      err_it = max(np.sum(tmp0, axis = 1))
      self.err_mean = np.mean(tmp0)


      if convergence != False:
        thefilename = convergence
        # thefilename = '/scratch/daint/petzip/ANN/1VERSION01/RESULTS/nn/convergence/Final_1'
        # thefilename = thefilename+self.projj+ self.rank +'.csv'
        if repeat % 2 == 0:
          if not os.path.exists(thefilename):
            with open(thefilename, 'a') as f_handle:
              writer = csv.writer(f_handle, delimiter = '\t', dialect = 'excel')
              writer.writerow(['Iter ', 'Errmax   ', 'Errmean   ', 'Pgood', 'ntype'])

          with open(thefilename, 'a') as f_handle:
            writer = csv.writer(f_handle, delimiter = '\t', dialect = 'excel')
            writer.writerow([('%4d' % repeat), ('%.3e' % err_it),('%.3e' % self.err_mean), ('%.2f' % self.P), self.ntypes])




      if repeat % 200 == 0 and print_intermediate == True:
        # print("\t \t -> inner_Iteration %3.f | max. Error: %.3e | mean Error: %.3e | P(good): %.3f |")%(repeat, err_it, self.err_mean, self.P)
        print(" -> inner_Iteration: %3d | max. Error: %.3e | mean Error: %.3e | P(good):  %.2f |" % (repeat, err_it, self.err_mean, self.P))
        # print((repeat, err_it, self.err_mean, self.P))
      else:
        pass
      self.repeat = repeat
      if self.err_mean <= max_err:
        break
      else: 
        pass
    #----------------------------------------------------------------------------
    #----------------------------------------------------------------------------
    self.tmpe1 = err_it #Make it accessable outside 
    self.tmpe2 = repeat #Make it accessable outside
    # print("+---------end-TRAINING--------------------+\n")

  def backpropagation(self, input, output_des):
    output = self.forwardpropagation(input)
    y      = output_des[:,None]
    self.intermed = y
    self.errors[-1] = self.primefunctions[-1](self.outputs[-1])*(output-y)
    self.backward_loop()


  def backward_loop(self):
    N=self.n_layers-2
    
    if self.Pcorr == True: 
      corrfac = self.corr_P()
    else: 
      corrfac = 1
    #----------------------------------------------------------------------------
    for i in range(N,0,-1):
      
      if self.adjuster == True: 
        if i == N: 
          learning_rate = self.adjust_learning_rate(self.learning_rate_out, self.tau_o)*corrfac
        else:
          learning_rate = self.adjust_learning_rate(self.learning_rate_hid, self.tau_h)*corrfac
      else:
        if i == N: 
          learning_rate = self.learning_rate_out
        else:
          learning_rate = self.learning_rate_hid
      self.p1 = learning_rate

      self.errors[i] = self.primefunctions[i](self.inputs[i])*self.weights[i].T.dot(self.errors[i+1])
      self.weights[i] = self.weights[i]-learning_rate*np.outer(self.errors[i+1],self.outputs[i])
      self.biases[i] = self.biases[i] - learning_rate*self.errors[i+1]
    self.weights[0] = self.weights[0]-learning_rate*np.outer(self.errors[1],self.outputs[0])
    self.biases[0] = self.biases[0] - learning_rate*self.errors[1]
    #----------------------------------------------------------------------------
  """Corrections/Adjustments to the learning rate"""

  def adjust_learning_rate(self, learning_rate, tau): 
    learning_rate = max(learning_rate * np.exp (-self.i/tau), self.eps)
    return learning_rate
  
  def corr_P(self):
    if self.q == 1: 
      tmp = -np.log(self.P)
    else: 
      tmp = -np.log((1-self.P))

    tmp2 = quality_crite(self.intermed)
    if tmp2 >= self.qcrit: 
      self.q = 1
    else: 
      self.q = 0

    self.P = ((self.P*self.i)+self.q)/(self.i+1)
    return tmp 

  """Calling the forward propagation, "Prediction"""
  def predict(self, input):
    n = len(input)
    m = self.sizes[-1]
    outp = np.ones((n,m))
    #----------------------------------------------------------------------------
    for i in range(len(input)):
      outp[i,:] = self.forwardpropagation(input[i])[:,0]
    #----------------------------------------------------------------------------
    return outp
    
  def save_network(self, Filename): 
    np.savez(Filename, self.Parameters, self.weights, self.biases)

  def weight_updater(self, weights): 
    self.weights = weights  

def load_network(Filename):
  File = np.load(Filename)
  Parameters = File['arr_0']
  BPN = Backpropagation_NN(Parameters)
  weights = File['arr_1']
  BPN.weight_updater(File['arr_1'])
  BPN.biases = File['arr_2']
  return BPN, weights

def quality_crite(input):
  quality = np.sqrt(input[0]**2+(input[1]-1.)**2)
  return quality

def out_quality(out, tresh):
  q_out = np.sqrt(out[:,0]**2 + (out[:,1]-1)**2)
  # print q_out
  q_out[q_out<=tresh] = 0 
  q_out[q_out>tresh] = 1
  # print q_out
  return q_out

def subsample(samples, output_des, size_subpackages):
  X = samples
  y = output_des
  N = X.shape[0]
  # print X.shape
  index = list(range(N))
  np.random.shuffle(index)
  packages = []
  tmp = []
  number_subsets = int(np.ceil(N/size_subpackages))
  for ii in range(0, number_subsets): 
    tmp.append(X[(ii*size_subpackages):(size_subpackages*(ii+1))])

  return tmp
  #   print tmp[1][0]  and print s[10] are now equivalent expressions, just subsampled! 
  #  


if __name__ == "__main__":
  raise ValueError("Please call this Neural Network from the Caller function!")
