"Output has to be s: list of arrays (inputs) and q: list of qualities"

from control.pathmaker import pathmaker
from mydetails import info
import numpy as np
import os, glob, sys

def prep_network(project='/test/', classifile = 'a11_classified_hand_Patrick', number_of_features=2, type2 = 'not ft', type0 = '/Training/', rank='aa', getgood = None):
    """
    Prepares the features (fft or wbfe) as input for the Neural Network

    Returns: 
    Array with features of dataset and
    Array with the classifications based on the classifile

    If getgood is enabled, s is also returned. S is the list of stations that were classified as good.
    For the getgood algorithm please see the function in ../ANN/get_good.py

    Hardcoded the number of features to 18 (2*9 because
    of the length of the WBFE extracted feature is 1 for every feature
    and two features were used)
    """
    # path_raw,path_postprocessed,path_postprocessed_meta,path_synthetics,path_classifications_hand,path_classifications_synthetic,representationpath, path_subsets = pathmaker(type='', project = project)
    path_postprocessed, representationpath, path_subsets, path_classifications_hand = pathmaker(type='', project = project)
    if type2 == 'ft':
        representationpath = representationpath+'/ft/'
    # print("\n \n ", path_subsets, type0)
    trainingsets_g = glob.glob(glob.glob(path_subsets+'/'+rank+'/'+type0)[0]+'/*'+classifile+'*good.npy')
    trainingsets_b = glob.glob(glob.glob(path_subsets+'/'+rank+'/'+type0)[0]+'/*'+classifile+'*bad.npy')
    # print((path_subsets+'/'+rank+'/'+type0)[0]+'/*'+classifile+'*good.npy')
    # print(trainingsets_g)
    g = np.load(trainingsets_g[0])
    b = np.load(trainingsets_b[0])
   
    lll = int(9*number_of_features)
    if type2=='ft': 
        lll = 18
    
    features = np.zeros((1,lll))
    
    quality = np.zeros((1,2))
    s = []
    for stations in g:
        tmp = np.load(representationpath+stations+'.npy').ravel()[None,0:lll]

        if type2 != 'ft': 
            if tmp[0][0] != 0 : 
                features = np.append(features,tmp, axis = 0)
                s.append([stations])
                quality = np.append(quality, np.array([[0.95,0.05]]), axis =0)

        else: 
            features = np.append(features,tmp, axis = 0)
            s.append([stations])
            quality = np.append(quality, np.array([[0.95,0.05]]), axis =0)


    for stations in b:
        tmp = np.load(representationpath+stations+'.npy').ravel()[None,0:lll]

        if type2 != 'ft': 
            if tmp[0][0] != 0 : 
                features = np.append(features,tmp, axis = 0)
                s.append([stations])
                quality = np.append(quality, np.array([[0.05,0.95]]), axis =0)

        else: 
            features = np.append(features,tmp, axis = 0)
            s.append([stations])
            quality = np.append(quality, np.array([[0.05,0.95]]), axis =0)

    if type2 != 'ft':
        for i in range(features.shape[-1]): 
            features[:,i] = features[:,i]/np.max(features[:,i])

    elif type2 =='ft':
        # Normalize Distance
        features[:,-1] = features[:,-1]/np.max(features[:,-1])
        features[:,-2] = features[:,-2]/np.max(features[:,-2])
        # Noramlize spectrum to maximum to maximum
        features[:,0:-2] = features[:,0:-2]/np.max(features[:,0:-2])

    if getgood != True:
        return features[1:], quality[1:]
    else:
        return features[1:], quality[1:], s


def prep_FT_network(project='XXXXX', classifile = 'XXXXX', rank='aa', getgood = None): 
    features, quality = prep_network(project=project, classifile = classifile, type2 ='ft', rank=rank, getgood=getgood)
    return features, quality


def call_network(project='XXXXXX', classifile='XXXXX', number_of_features=2, type2='notft', type0='/Test/', rank='aa', getgood = None):
    if getgood != True:
        features, quality = prep_network(project=project, classifile = classifile, type2 =type2, type0=type0, rank=rank, getgood=getgood)
        return features, quality
    if getgood == True:
        features, quality, s = prep_network(project=project, classifile = classifile, type2 =type2, type0=type0, rank=rank, getgood=getgood)
        return features, quality, s


def call_FT_network(project='XXXXXX', classifile = 'XXXXXX', rank='aa', getgood = None):
    features, quality = call_network(project=project, classifile=classifile, type2='ft', rank=rank, getgood=getgood)
    return features[1:], quality[1:]


#
# feat_, _ = prep_network(project='/test/', classifile = 'test_visualclassification_Patrick', rank="1")
# print(np.sum(feat_[0]))
#
# feat_, _ = prep_FT_network(project='/test/', classifile = 'test_visualclassification_Patrick', rank="1")
# print(np.sum(feat_[0]))
