import numpy as np
import matplotlib.pyplot as plt
from obspy import read, Trace, Stream
from obspy import UTCDateTime as utc
import os
import glob
import datetime
import  scipy.signal as signal

def get_time(trace): 
        l = np.arange(len(trace.data)/2+1)
        l = np.append(-1*l[-1:0:-1], l)
        time = l*trace.stats.sampling_rate
        return time

def get_time2(trace): 
        l = np.arange(len(trace.data))
        time = l*trace.stats.sampling_rate
        return time
def get_time3(trace): 
    l = np.arange(len(trace)*trace.stats.sampling_rate)
    # np.arange(len(self.stream_[0].data))*self.stream_[0].stats.sampling_rate

def get_info_from_sac(trace):
    # For the Correlations
    _id = str(trace.id)
    stat = trace.stats
    s1   = _id + '-'
    d1   = ' '+str(stat.sac.dist).strip()+' '
    s2   = stat.sac.kuser0.strip()+'.'+stat.sac.kevnm.strip()+'.'+\
            stat.sac.kuser1.strip()+'.'+ (stat.sac.kuser2).strip()
    return 'Station:  '+s1+s2

def get_info_from_raw(trace):
    stat = trace.stats
    # For the raw data:
    name = ('%s.%s.%s.%s') % (stat.network, stat.station, stat.location, stat.channel)
    return 'Station:  '+name

def prep_plot(trace):
    t_start = utc(trace.stats.starttime).datetime
    date_list = [t_start+datetime.timedelta(seconds=x) for x in range(0,len(trace))]
    return date_list, trace

def plott(x,y,type='trace',type2='raw', subfigure = False, fsize = (10,10), title=' ', savepath = '/home/patrick/1Thesis/track_files/pictures/2corr/', i=99, basename = 'asd', velocity = 3800.):
    plt.locator_params(nbins=5)
    if type2 == 'raw' or 'rawpp': 
        info = get_info_from_raw
        xlab = 'Time after starttime [d]'
        stype='.png'

    if type2 == 'corr' or 'window' or 'feat': 
        info = get_info_from_raw
        xlab = 'Lag Time [s]'
        stype='.eps'

    if type2 == 'feat':
        dist = y.stats.sac.dist / velocity

    if type == 'trace':
        label = info(y)
        if type2 == 'corr': 
            x = get_time(y)
        if type2 in ['raw','rawpp']:
            label = label+('\nstarttime %s')%(str(y.stats.starttime.ctime()))
            x = get_time2(y)/86400.
        if type2 in ['window','feat']: 
            x = get_time2(y)


        y = y.data
        ma = np.max(np.abs(y))
        y = y/ma

        f, ax = plt.subplots(1,1, figsize = fsize)

        # max_xticks = 5
        # xloc = plt.MaxNLocator(max_xticks)
        # ax.xaxis.set_major_locator(xloc)

        ax.plot(x,y, label=label, color='black')

        if type2 == 'feat':
            representationpath = '/home/patrick/1Thesis/wavemanipulation/2classification/representations/a11/'
            velocity = 3800.
            ax.axvline(dist, color='r',linewidth='2', label = 'estimated arrival')
            variables = np.load(representationpath+basename+'.npy')
            rec = reconstruct(variables, wavetype='morlet', number_of_features=2)
            ax.plot(x, rec, color = 'green', label='Extracted Feature, n=2')

        ax.legend(loc = 4, borderaxespad=0., fontsize = 12)
        # ax.set_xticks([x[0],x[-1]])
        # ax.yaxis.set_major_formatter(plt.NullFormatter())
        # ax.set_yticks([-1,0,1])
        
        ax.set_xlabel(xlab)
        ax.set_ylabel('Normalized Amplitude')
        ax.set_title(title)
        ax.set_xlim([x[0],x[-1]])
        ax.set_ylim([np.min(y)*1.3, np.max(y)*1.3])
        # ax.set_ylim([y[]])
        # f.savefig(savepath+type2+label.split(':')[1].split('\n')[0]+stype)
        f.savefig(savepath+type2+label.split(':')[1].split('\n')[0]+'.png')


    elif type == 'stream':
        i = 0     
        tmp = len(y)
        if subfigure == True:
            f,axarr = plt.subplots(tmp,1)
            for trace in y:
                x,y = prep_plot(trace)
                label = info(trace)
                if type2 == 'raw': 
                    time = get_time2(y)
                elif type2 == 'corr': 
                    time = get_time(y)

                axarr[i].plot(time, trace.data, label=label, color='black')
                axarr[i].legend(loc = 2, fontsize = "small", borderaxespad=0.)
                axarr[i].yaxis.set_major_formatter(plt.NullFormatter())
                i+=1
            # plt.subplots_adjust(wspace=0, hspace=0)
            axarr[0].set_title(title)
            axarr[-1].set_xlabel(xlab)
            axarr[-1].set_ylabel('Normalized Amplitude')
            f.savefig(savepath+str(i))

    plt.close(f)


def reconstruct(variables, wavetype='morlet', number_of_features=2):
        tmp   = np.zeros(variables[0][-2])
        recon = tmp.copy()
        for i in range(0, number_of_features):
            tmp2  = tmp.copy()
            starttime = variables[i][0]
            duration  = variables[i][1]
            crossings = variables[i][4]
            polarity  = variables[i][3]
            amp_ratio = variables[i][-1]
            residuals = variables[i][-3]

            temp_wave = np.real(signal.morlet(len(tmp[starttime:(starttime+duration)]), 
                                              w=crossings, s=(1/2.), complete=True))

            if polarity == 1:
                tmp2[starttime:(starttime+duration)] = temp_wave/np.max(temp_wave)*amp_ratio
                recon = np.add(recon, tmp2)
            else:
                tmp2[starttime:(starttime+duration)] = -temp_wave/np.max(-temp_wave)*amp_ratio
                recon = np.add(recon, tmp2)
        return recon





f_dir0 = '/home/patrick/1Thesis/track_files/'
picpath = '/home/patrick/1Thesis/track_files/pictures/'
do_raw = True
do_preproc = True
do_corr = False
do_corr_w = False
do_corr_fe = False


arr = np.array(['0raw', '1preproc', '2corr', '3corr_postproc', '5feature_extr'])
arr2 = (['good', 'bad'])
fsize = (12,5)













if do_raw ==True: 
    print 'RAW RUNNING...'
    # RAW DATA
    f_dir = f_dir0+arr[0]+'/*/'
    for i in range(len(glob.glob(f_dir))):
    # for i in range(1):
        a = glob.glob(f_dir)[i]+'*'
        stream = read(a)
        stream.normalize()
        x,y = prep_plot(stream[0])
        plott(x,stream[0],type='trace', type2='raw', subfigure = True, fsize = fsize, title = ' Unprocessed Raw  ', savepath = '/home/patrick/1Thesis/track_files/pictures/99', i=i)

if do_preproc ==True: 
    print 'PREPROC RUNNING'
    f_dir = f_dir0+arr[1]+'/*/'
    # RAW DATA
    for i in range(len(glob.glob(f_dir))):
    # for i in range(1):
        a = glob.glob(f_dir)[i]+'*'
        stream = read(a)
        stream.normalize()
        x,y = prep_plot(stream[0])
        plott(x,stream[0],type='trace', type2='rawpp', subfigure = True, fsize = fsize, title = ' Preprocessed Raw  ', savepath = '/home/patrick/1Thesis/track_files/pictures/99', i=i)

if do_corr ==True:
    print 'CORR RUNNING'
    f_dir = f_dir0+arr[2]+'/*'
    for i in range(len(glob.glob(f_dir))):
    # for i in range(1):
        a = glob.glob(f_dir)[i]
        # print a
        stream = read(a)
        stream.normalize()
        x,y = prep_plot(stream[0])
        plott(x,stream[0], type='trace', type2='corr', subfigure = True, fsize = fsize, title = ' Correlations ',savepath = '/home/patrick/1Thesis/track_files/pictures/99', i=i)

if do_corr_w ==True:
    print 'CORRw RUNNING'
    f_dir = f_dir0+arr[3]+'/*'
    for i in range(len(glob.glob(f_dir))):
    # for i in range(1):
        a = glob.glob(f_dir)[i]
        stream = read(a)
        stream.normalize()
        x,y = prep_plot(stream[0])
        plott(x,stream[0], type='trace', type2='window', subfigure = True, fsize = fsize, title = 'Windowed Correlations ',savepath = '/home/patrick/1Thesis/track_files/pictures/99', i=i)

if do_corr_fe ==True:
    print 'CORRfe RUNNING'
    f_dir = f_dir0+arr[3]+'/*'
    for i in range(len(glob.glob(f_dir))):
        a = glob.glob(f_dir)[i]
        # print os.path.basename(a)
        stream = read(a)
        stream.normalize()
        x,y = prep_plot(stream[0])
        plott(x,stream[0], type='trace', type2='feat', subfigure = True, fsize = fsize, title = 'Windowed Correlations ',savepath = '/home/patrick/1Thesis/track_files/pictures/99', i=i, basename = os.path.basename(a))






