
from control.pathmaker import pathmaker

from obspy import read, Trace
import numpy as np
import os, glob
import random
import pandas as pd
import matplotlib.pyplot as plt
# from ANN import

import math as mt

class ostream(object):
    """
    Object class for the stream, initially reads it in
    """
    def __init__(self, originpath, number_of_distance_classes=0, constraint = None):
        """
        Originpath is the path to read in, including the *.datatype.
        Number of distance classes was decided to be not implemented, so therefore st to 0.
        constraint can be None, 'good' or 'bad' to read in all traces, 
        the good ones or the bad ones (as classified by hand)
        """
        self.constraint = constraint
        self.n = number_of_distance_classes
        self.reader(originpath)
        self.stream_ = self.stream_original.copy()

    def reader(self, path, normalize =True):
        # print path
        """
        Read in the files from the path,
        if normalized: Normalize every trace after reading in

        The class variables are initialized: 
        _.stream_           Obspy stream containing the read in traces
        _.corrlist          List with the names of the read in Files 
        _.stream_original   Copy of the stream as backup. Might not be needed.  
        """
        # print 'running ostream.reader()'
        # print 'Initializing the reader'
        # print 'Constraint for reading in: ', self.constraint
        # print 'Path to read in: ', path
        self.corrlist = []
        i = 0
        if self.constraint == None:
            for corr in glob.glob(path): 
                if i == 0: 
                    stream_ = read(corr)
                else: 
                    trace = read(corr)
                    stream_ = stream_.__add__(trace)
                self.corrlist.append(os.path.basename(corr))
                i+=1
        else: 
            for i in range(len(self.constraint)): 
                if i == 0: 
                    stream_ = read(path+str(self.constraint[i]))
                else: 
                    trace = read(path+str(self.constraint[i]))
                    stream_ = stream_.__add__(trace)
                self.corrlist.append(self.constraint[i])
                i+=1
        self.stream_ = stream_

        if normalize == True:
            
            stream_ = stream_.normalize(global_max=False)
            self.normalize()
        self.stream_original = stream_


    def normalize(self, type='absolute'):
        """
        Normalizin all the traces within self.stream_
        """ 
        # print 'running ostream.normalize()'
        if type == 'positive_max': 
            for trace in self.stream_: 
                trace.data = trace.data/np.max(trace.data)
        elif type == 'absolute':  
            self.stream_ = self.stream_.normalize(global_max=False)
        else: 
            print('No valid Normalization chosen, please revisit. \nUsable options: absolute or positive_max')

    def distance_attribute(self): 
        """
        Get the .stats.distance attribute for every Trace, because the .SAC header has saved it as 
        trace.stats.sac.dist. Needed for example to use obspy.stream.plot(type='section')
        """
        # print 'running ostream.distance_attribute()'
        try: 
            for trace in self.stream_: 
                trace.stats.distance = trace.stats.sac.dist
        except: 
            print("COULD NOT OBTAIN DISTANCE INFORMATION! \n MAYBE DUE TO FILETYPE (only .SAC supported)")

    def get_max_dist(self):
        """
        Get class variable 
        _.maxdist 
        as maximum distance between correlations for a whole stream. Might be needed for plotting functions
        and basic statistics
        """
        # print 'running ostream.get_max_dist()'
        tmp = 0
        for trace in self.stream_: 
            if trace.stats.sac.dist > tmp: 
                tmp = trace.stats.sac.dist
        self.maxdist = tmp

    def get_time(self): 
        """
        Get class variable 
        _.time
        Creates the time array for plotting. Takes lenth of correlation and divided by two and 
        uses sampling rate to create array with datapoints for x-axis
        """
        # print 'running ostream.get_time()'
        l = np.arange(len(self.stream_[0].data)/2+1)
        l = np.append(-1*l[-1:0:-1], l)
        self.time = l*self.stream_[0].stats.sampling_rate

    def get_time_causal(self): 
        """
        Overwrites the ostream.time variable with only the causal part
        """
        self.time = np.arange(len(self.stream_[0].data))*self.stream_[0].stats.sampling_rate
        
    def plot_stream2(self, scale_distance=0.7*1e5, speed=3900., 
                     annotate=False, 
                     which='orig', nfeat=4, 
                     subf = True, streamarr = ['yourfileshere'], 
                     headers=['here'], features=['here'], frequencies=['here']):

        """
        Function to visualize a full stream with a section-plot
        scale_distance  is just a scaling factor, you can leave it hardcoded. 
                        Might be adjusted to increase the signal peaks
        which   is not implemented
        nfeat   is the number of features that should be plottet in case features are plotted
        # fband   is the string for the title of the plot, the frequency band.
        subf    should always be True, creates the subfigure
        streamarr   is a list with streams to plot. The length defines the amount of subplots
        headers     is a list with the titles for the plot
        features    is a list with the number of features for the header
        frequencies is a list with the correct frequency bands for the subplot titles

        Returns: 
        A figure f and the axes array axarr.
        Figure then can be saved with pyplot.figure.savefig() or displayed with pyplot.show()

        Do not forget to close the figure if not used anymore!

        """
        # print 'running plot_stream2()'
        f, axarr = plt.subplots(len(streamarr), 1, sharex=True, sharey=True, figsize=(15,15), dpi=600)
        i = 0
        
        self.get_time_causal()
            
        
        if subf == True:

            for i in range(len(streamarr)):
                if len(streamarr)<=1: 
                    ax = axarr
                else: 
                    ax = axarr[i]
                stream = streamarr[i]
                f.hold(True)
                for trace in stream:
                    label_ = self.get_info_from_sac(trace)
                    ax.plot((trace.data*scale_distance+trace.stats.sac.dist)/1000, self.time, color='black', alpha=0.6)
                ax.set_ylabel('Lag Time [s]')
                
                ax.grid()
                if speed != None: 
                    ax.plot(self.time[self.time>0]*speed/1000, self.time[self.time>0], 'r--', linewidth =1.5, alpha =.7, label=('Velocity %.0fm/s')%(speed))
                    ax.set_title(headers[i]+''+str(features[i])+'; '+'Bandwidth='+frequencies[i])

        ax.set_xlabel('Interstation Distance [km]')
        plt.subplots_adjust(wspace=0.2, hspace=0.2)
        # axarr[0].set_title('Noise Correlations, N=')
        plt.legend(loc=4, fontsize = "small")
        
        plt.ylim(0, max(self.time))
        # plt.xlim(-10e1/1000, 1e7/1000)
        plt.gca().invert_yaxis()


        return f, axarr# plt.show()

    def get_info_from_sac(self, trace):
        """
        Get the two stations of a crosscorrelation as a String in the Format
        network.station - network.station
        """
        # print "running ostream.get_info_from_sac"
        _id = str(trace.id)
        stat = trace.stats
        s1   = _id + '-'
        # d1   = ' '+str(stat.sac.dist).strip()+' '
        s2   = stat.sac.kuser0.strip()+'.'+stat.sac.kevnm.strip()+'.'+\
                stat.sac.kuser1.strip()+'.'+ (stat.sac.kuser2).strip()
        # print s1+s2
        return s1+s2

    def get_info_from_sac_acausal(self, trace):
        """
        Gets the SAC info for the acausal parts of the stream
        """
        # print "running ostream.get_info_from_sac_acausal"
        _id = str(trace.id)
        stat = trace.stats
        s1   = _id
        # d1   = ' '+str(stat.sac.dist).strip()+' '
        s2   = stat.sac.kuser0.strip()+'.'+stat.sac.kevnm.strip()+'.'+\
                stat.sac.kuser1.strip()+'.'+ (stat.sac.kuser2).strip()+'-'
        # print s2+s1
        return s2+s1

    def load_subset(self,  number, which = 'Training', form='good', project = '/a11/', classifile = 'a11_classified_hand_Laura'):
        """
        Loader of subsets in the subset path specified by pathmaker
        """
        # print "running ostream.load_subset()"
        _,_,_,_,_,_,_, path_subsets = pathmaker(type='*.SAC', project = project)
        
        project = '/'+project.split('/')[1]+'-'
        path_sublist = path_subsets+which+project+str(number)+classifile+form
        self.sublist = np.load(path_sublist+'.npy')
        # print self.sublist
        return self.sublist

    def subsets_v2(self,number='all', which = 'Training', form='good', project = '/a11/',classifile = 'test', number2=2 , rank ='0'): 
        """
        Function to create subsets (training and test) from a stream
        number: Either 'all' that splits the dataset in half (good half, bad half)
                or a number that then defines the size of the good and bad traces.
                The total size of the dataset then will be 2x number
        which:  Training or test, string
        form:   good or bad
        classifile : the file containing the classifications
        rank:   rank to save the subsets in
        """
        # print "running ostream.subsets_v2()"
        _, _, path_subsets, _ = pathmaker(type='*.SAC', project=project)
        if which == 'Training':
            self.sublist = self.corrlist
            
        if which in ['Training', 'Test']: 
            if number == 'all':
                number = int(len(self.corrlist)/2)
                # n = mt.ceil(len(self.corrlist)/2)
                # if which == 'Training':
                #     self.sublist = self.corrlist[0:n]
                # # if which == 'Test':
                # #     self.sublist = self.corrlist
                # if which == 'Test': 
                #     self.sublist = self.corrlist[n:-1]
                #     # self.sublist = self.corrlist
            else:
                number =int(number)
            try:
                if which == 'Training':
                    self.corrlist2 = np.random.permutation(self.corrlist)
                    self.sublist = self.corrlist2[0:number]
                    print('Training:\t #' +form+ '\t', len(self.sublist))
                if which == 'Test': 
                    self.sublist = self.corrlist2[number:]
                    print('Testing :\t #' +form+ '\t', len(self.sublist))
                # if which == 'Test':   
                    # self.sublist = self.corrlist2

            except ValueError as e: 
                print(e)
                print('recreating subset with replace=True')
                self.sublist = np.random.choice(self.corrlist, size=(number))

        project = '/'+project.split('/')[1]+'-'
        path_sublist = path_subsets+'/'+rank+'/'+which
        filestodel = glob.glob(path_sublist+'/*')

        if len(filestodel)>=2:
            for f in filestodel:    
                os.remove(f)
        path_sublist = path_sublist+project+str(number)+classifile+form

        np.save(path_sublist, self.sublist)

def subset_creator(ng=100, nb=100, project ='/a11/', classificationfile = 'a11_classified_hand_Laura.npy', number2 = 2, rank ='0'):
    """
    Creates the subsets for a project (train and test)
    classificationfile is the name of the file with the hand classifications
    number2 is not implemented anymore.
    rank is the rank to save the subset in
    """
    # print "running ostream.subset_creator()"
    path_postprocessed, features, subsets, path_classifications_hand = pathmaker(type='*.SAC', project = project)
    # path_raw,path_postprocessed,path_postprocessed_meta,path_synthetics,path_classifications_hand,path_classifications_synthetic,representationpath, path_subsets = pathmaker(type='*.SAC', project = project)
    classi = 'hand'
    o = ostream(path_postprocessed)
    classified = np.load(path_classifications_hand + classificationfile)
    c_df = pd.DataFrame(classified)
    c_df = c_df.drop_duplicates()
    c_df = c_df.sort_values(0)

    cl_df = pd.DataFrame(o.corrlist)
    cl_df = cl_df.sort_values(0)

    classified = c_df.as_matrix()
    o.corrlist = cl_df.as_matrix().flatten()
    backuplist = o.corrlist.copy()


    whichones = 'good'
    if whichones == 'good':
        x = [np.where(classified[:,1] == '1')]
        x = np.ravel(x).tolist()
        o.corrlist = np.array(o.corrlist)[[x]].tolist()
        np.random.shuffle(o.corrlist)
        o.subsets_v2(ng,which = 'Training', form=whichones, project=project, classifile = classificationfile.split('.npy')[0], number2=2, rank=rank)
        o.subsets_v2(ng,which = 'Test', form=whichones, project=project, classifile = classificationfile.split('.npy')[0], number2=2, rank=rank)
    
    np.random.seed()
    whichones = 'bad'
    if whichones =='bad':
        o.corrlist = backuplist
        x = [np.where(classified[:,1] == '0')]
        x = np.ravel(x).tolist()
        o.corrlist = np.array(o.corrlist)[[x]].tolist()
        np.random.shuffle(o.corrlist)
        o.subsets_v2(nb,which = 'Training', form=whichones,project=project, classifile = classificationfile.split('.npy')[0], number2=2, rank=rank)
        o.subsets_v2(nb,which = 'Test', form=whichones,project=project, classifile = classificationfile.split('.npy')[0], number2=2, rank=rank)

if __name__ == '__main__': 
    raise ValueError('If you want to run create_subsets_full, please call subsetcreator.py')
    
    
