from control.basic_functions import *

import numpy as np
import os, glob, sys

import matplotlib.pyplot as plt

import  scipy as sc
import  scipy.signal as signal
import  scipy.integrate as integrate
from    scipy.fftpack import fft
from    scipy.integrate import simps
from scipy import interpolate
from control.pathmaker import pathmaker
from obspy import read, Trace, Stream
from mydetails import info


def FT(data):
    #print("Running FT()")
    """
    Returns the FFT of data
    """
    return (np.abs(np.fft.fft(data))**2)[0:len(data)//2]

def FTx(data): 
    #print "running FTx()"
    """
    Returns the frequencies of the FFT from the data
    """
    return np.fft.fftfreq(data.size, 1.)[0:len(data)//2]

def Resample(ft, periods, pmin=20, pmax=130, intervals = 18, distance =0):

    """
    Resample the ft (from FT) with the('periods') (from FTx)
    periods as number of periods within the period bandwidth from
    pmin to 
    pmax

    returns: 
    nx:     The x-values, the periods for the resampled FFT-spectrum
    ny:     The resampled FFT-spectrum
    """
    # FT from FT
    intervals = intervals-1
    #print "running Resample()"
    s = interpolate.interp1d(periods,ft)
    idx = np.all([periods >= pmin, periods <= pmax], axis=0)
    x = periods[idx]
    y = ft[idx]

    xnew = np.linspace(pmin,pmax,num=10000, endpoint=True)
    ints = np.linspace(pmin,pmax, num=intervals, endpoint=True)

    delta = (1./pmin - 1./pmax)/intervals
    alpha = simps(s(xnew), xnew)
    nx = []
    ny = []
    for i in range(len(ints)-1):

        min = ints[i]
        max = ints[i+1]
        # #print
        idx = np.all([xnew>=min, xnew <=max], axis=0)
        tmpx = xnew[idx]
        tmpy = s(tmpx)
        ttmpy = simps(tmpy, x=tmpx)/(alpha*delta)
        nx.append(tmpx[0]+(tmpx[-1]-tmpx[0])/2)
        ny.append(ttmpy)
    ny.append(alpha)
    ny.append(distance)
    
    # #print distance, alpha
    return nx, ny

def resampler(data, pmin=30, pmax=130, intervals = 18, return_orig=False): 
    """
    Caller for the Resampler of the DATA within the period band from pmin to pmax and the number of intervals.

    If return_orig is True, then also the original (not resampled) FFT with the periods are returned.
    """
    distance = data.stats.sac.dist
    data = data.data
    
    #print 'running resampler()'
    ft = FT(data)[1:]
    ft = ft/np.max(ft)
    periods = 1./FTx(data)[1:]
    # returns periods_resampled and FT_resampled
    if return_orig ==True: 
        x,y = Resample(ft, periods, pmin=pmin, pmax=pmax, intervals=intervals, distance=distance)
        return periods, ft, x, y
    else: 
        return Resample(ft, periods, pmin=pmin, pmax=pmax, intervals=intervals, distance=distance)

class featureextracter(ostream):
    """
    ostream class for the featureextracter
    """
    def extract_feature(self, trace, distance='none', number_of_features=2, prevlist=[], cros_idx =10, velo=3800.):
        """
        Wavelet Based feature extracter algorithm based on the scipy signal morlet wavelet.
        For every 
        trace within a stream, 
        number of features,
        yero crossings from 2 to 10

        prevlist stores the features for every iteration

        returns: 
        out_representation = np.array([[out_ts, out_dur, out_dratio, out_phase, out_cidx, out_fit, out_res, len(y), _m]])
        array with number of features as arrays within.
        the order is: 
        -starttime
        -duration (duration of the morlet wavelet)
        -distance ratio x[np.abs(y_)>0.1][0]*velo/distance
        -phase (1 / -1)
        -zero-crossing-index
        -fit: out_fit = np.sum(np.abs(y[x[zmp:zpp]] - np.real(tmp_y)))/len(tmp_y)
        -residual np.sum(np.abs((y-y_)))/len(y)
        -length (number of datapoints total)
        -amplitude ratio
        """
        n = number_of_features
        y = trace
        # distance = trace.stats.sac.dist
        n_iter      = 0.
        c_iter      = 0.
        comp_iter   = 0.

        out_ts      = [] # Starttimes
        out_dur     = [] # Durations
        out_dratio  = [] # Duration Ratio
        out_n       = [] # Number of Datapoints
        out_cidx    = [] # Zero Crossing index
        out_fit     = [] # Fit of current waveform
        out_res     = [] # Residuals after current waveform
        out_phase   = [] # Polarity of waveform

        zero_crossings = np.where(np.diff(np.signbit(y)))[0]
        cros_idx_r = cros_idx
        cros_idx_l = cros_idx

        x   = np.arange(len(y))
        tmp = y[np.where((x>zero_crossings[2]) & (x < zero_crossings[-2]))]
        m   = np.max(tmp)
        # #print len(x)
        non = None                  #For while-loop
        ii  = 0
        # try:
        while non == None:

            for i in range(2,cros_idx_l):

                for iii in range(-40,40,1):

                    for compress in range(-5,5,1): 

                        try:

                            zcp3 = zero_crossings[zero_crossings>x[y==m]][i]+iii+compress
                            zcm3 = zero_crossings[zero_crossings<x[y==m]][-i]+iii-compress
                            tmp_y0 = y[x[zcm3:zcp3]]
                            tmp_p_ = np.real(signal.morlet(len(y[x[zcm3:zcp3]]),w=i, s=(1/2.), complete=True))

                            tmp_p = tmp_p_ / np.max(tmp_p_) * m
                            tmp_n = -1*tmp_p_ / np.max(-tmp_p_) *m

                            
                            if (np.sum(np.abs(tmp_y0 - tmp_p)) < np.sum(np.abs(tmp_y0 - tmp_n))):

                                tmp = tmp_p
                                tmp_p = 1

                            else:

                                tmp = tmp_n
                                tmp_p = 0

                            if ii > 0:

                                if prevmisfit < (np.sum(np.abs(tmp-tmp_y0)))/len(tmp): 
                                    tmp = tmp_prev
                                    tmp_y0 = tmp0prev

                                else:

                                    _m = m
                                    n_iter = iii
                                    c_iter = i
                                    comp_iter = compress
                                    zpp = zcp3
                                    zmp = zcm3
                                    tmp_prev = tmp.copy()
                                    tmp0prev = tmp_y0.copy()
                                    prevmisfit = np.sum(np.abs(tmp-tmp_y0))/len(tmp)
                                    p_iter = tmp_p

                            else:

                                _m = m
                                n_iter = iii
                                c_iter = i
                                comp_iter = compress
                                tmp_prev = tmp.copy()
                                tmp0prev = tmp_y0.copy()
                                prevmisfit = np.sum(np.abs(tmp_prev-tmp0prev))/len(tmp)
                                zpp = zcp3
                                zmp = zcm3
                                p_iter = tmp_p

                            ii = ii+1

                        except ValueError:
                            pass
                            # #print "ValueError"

                        except IndexError:
                            # zpp = 0
                            # zmp = 1
                            pass
                            # #print "IndexError"

            non = 0
            tmp_y = tmp
            tmp_x = x[x[zmp:zpp]]
            y_ = np.zeros_like(y)
            y_[x[zmp:zpp]] = np.real(tmp_y)
            
            out_ts = zmp
            out_dur = zpp-zmp
            velo = velo
            out_dratio = x[np.abs(y_)>0.1][0]*velo/distance
            out_phase = p_iter
            out_fit = np.sum(np.abs(y[x[zmp:zpp]] - np.real(tmp_y)))/len(tmp_y)
            out_res = np.sum(np.abs((y-y_)))/len(y)
            out_cidx = c_iter
            out_representation = np.array([[out_ts, out_dur, out_dratio, out_phase, out_cidx, out_fit, out_res, len(y), _m]])
            
            if prevlist != []: 
                out_representation = np.append(prevlist, out_representation, axis = 0)
            else:
                pass


        if n > 1:
            n = n - 1
            tmp = y_
            y = y - y_
            y_p = y_.copy()
            y_, out_representation = self.extract_feature(y,distance=distance, number_of_features=n, prevlist=out_representation)
            y_ = y_+y_p

        else: 

            return y_, out_representation

        return y_, out_representation

    def reconstruct(self, variables, wavetype='morlet', number_of_features=2):
        """
        Reconstruct the trace with the extracted wavelet for the number of features.
        """
        # print(variables)
        tmp   = np.zeros(int(variables[0][-2]))
        recon = tmp.copy()
        for i in range(0, number_of_features):
            tmp2  = tmp.copy()
            starttime = int(variables[i][0])
            duration  = int(variables[i][1])
            crossings = int(variables[i][4])
            polarity  = variables[i][3]
            amp_ratio = variables[i][-1]
            residuals = variables[i][-3]

            temp_wave = np.real(signal.morlet(len(tmp[starttime:(starttime+duration)]), 
                                              w=crossings, s=(1/2.), complete=True))

            if polarity == 1:
                tmp2[starttime:(starttime+duration)] = temp_wave/np.max(temp_wave)*amp_ratio
                recon = np.add(recon, tmp2)
            else:
                tmp2[starttime:(starttime+duration)] = -temp_wave/np.max(-temp_wave)*amp_ratio
                recon = np.add(recon, tmp2)
        return recon

        # out_representation = np.array([[out_ts, out_dur, out_dratio, out_phase, 
        # out_cidx, out_fit, out_res, len(y), _m]])
        

    def plot_reconstructions(self):
        """
        Plot the reconstructions together with the power spectra (second column)
        Returns a pyplot figure that then can be saved.
        """
        n = len(self.stream_)
        # f, axarr = plt.subplots(n,3,sharex='col')
        f, axarr = plt.subplots(n,2,sharex='col', figsize = (15,15), dpi=600)
        # f, axarr = plt.subplots(n,2,sharex='col', figsize = (15,15))
        f.subplots_adjust(hspace=0.03)
        f.subplots_adjust(wspace=0.03)
        f.hold(True)
        i = 0
        for trace in self.stream_:
            tr = trace.data
            
            ftx1, fty1, ftx2, fty2 = resampler(trace, pmin=50, pmax=110, intervals = 18, return_orig=True)

            name = self.corrlist[i]
            distance = trace.stats.sac.dist

            resid = tr - self.reconstructions[i]
            
            axarr[i,0].plot(tr, label = name + str(distance), c = 'black', linewidth=1.5)
            
            axarr[i,0].plot(self.reconstructions[i], 'r', label='WBFE Waveforms $W_n$ for two iterations', linewidth=1.)
            axarr[i,0].set_ylim([-1.2, 1.3])
            axarr[i,0].set_xlim([0, len(tr)])

            if i!=len(self.stream_)-1:
                axarr[i,0].yaxis.set_major_formatter(plt.NullFormatter())
                # axarr[i,0].xaxis.set_major_formatter(plt.NullFormatter())
            axarr[i,0].legend(loc = 2, fontsize = "small", borderaxespad=0., frameon=False)
            # axarr[i,0].grid()

            axarr[i,1].plot(ftx1, fty1, label='Power Spectrum $f(\omega)$ of correlation '+name, color='black', linewidth=1.5)
            axarr[i,1].plot(ftx2, fty2[0:-2]/np.max(fty2[0:-2]), label='N-point Representation $P_n$', color='red', linewidth=1., marker='.')
            #print len(ftx2),len(self.ftfeatures[i])

            # axarr[i,1].plot(ftx2, self.ftfeatures[i][0:-2]/np.max(self.ftfeatures[i][0:-2]), 'b--', label='FFT as stored', marker='.')
            # axarr[i,1].set_xlim([20,130])
            axarr[i,1].set_xlim([40,120])
            axarr[i,1].set_ylim([0,1.2])
            iiii = 50
            while iiii <= 111: 
                axarr[i,1].axvline(x=iiii, c='gray')
                iiii += (ftx2[1]-ftx2[0])
            if i!=len(self.stream_)-1: 
                axarr[i,1].yaxis.set_major_formatter(plt.NullFormatter())
                # axarr[i,1].xaxis.set_major_formatter(plt.NullFormatter())

            axarr[i,1].legend(loc = 2, fontsize = "small", borderaxespad=0., frameon=False)
            # axarr[i,1].grid()
            axarr[i,1].yaxis.tick_right()



            # axarr[i,2].plot(resid, label=('mean residuals %.6f') %(self.features[i][-1][-3]))
            # axarr[i,2].legend(loc = 2, fontsize = "small", borderaxespad=0.)
            # axarr[i,2].set_ylim([-1.2, 1.3])
            # # axarr[i,2].yaxis.set_major_formatter(plt.NullFormatter())
            # # axarr[i,2].xaxis.set_major_formatter(plt.NullFormatter())
            # axarr[i,2].legend(loc = 4, fontsize = "small", borderaxespad=0.)
            i+=1

        axarr[0,0].set_title('Normalized Seismogram and Features')
        axarr[0,1].set_title('Normalized Power spectrum')
        # axarr[0,2].set_title('Residuals')

        axarr[-1,0].set_xlabel('Lag Time [s]')
        axarr[-1,1].set_xlabel('Period [s]')
        # axarr[-1,2].set_xlabel('Lag Time [s]')
        return f
        # plt.show()

    def plot_reconstructions3(self):
        """
        Plot the reconstructions together with the power spectra (second column)
        Returns a pyplot figure that then can be saved.
        """
        n = len(self.stream_)
        # f, axarr = plt.subplots(n,3,sharex='col')
        # f, axarr = plt.subplots(1,sharex='col', figsize = (15,15), dpi=600)
        f, axarr = plt.subplots(1,sharex='col', figsize = (18,6))
        # f, axarr = plt.subplots(n,2,sharex='col', figsize = (15,15))
        f.subplots_adjust(hspace=0.03)
        f.subplots_adjust(wspace=0.03)
        f.hold(True)
        i = 0
        for trace in self.stream_:
            tr = trace.data
            
            ftx1, fty1, ftx2, fty2 = resampler(trace, pmin=50, pmax=110, intervals = 18, return_orig=True)

            name = self.corrlist[i]
            distance = trace.stats.sac.dist

            # resid = tr - self.reconstructions[i]
            
            # axarr[i,0].plot(tr, label = name + str(distance), c = 'black')
            
            # axarr[i,0].plot(self.reconstructions[i], 'r--', label='extracted feature')
            # axarr[i,0].set_ylim([-1.2, 1.3])
            # axarr[i,0].set_xlim([0, len(tr)])

            axarr.plot(ftx1, fty1, label='Power Spectrum $f(\omega)$ of correlation '+name, color='black', linewidth=2.)
            axarr.plot(ftx2, fty2[0:-2]/np.max(fty2[0:-2]), label='N-point Representation $P_n$', color='red', marker='.', linewidth=1.5)
            # #print ftx2
            # #print ftx2[1]-ftx2[0], ftx2[-1]-ftx2[-2]
            i = 50
            while i <= 111: 
                axarr.axvline(x=i, c='gray')
                i += (ftx2[1]-ftx2[0])
            # axarr.axvline(x=50, c='red')
            # axarr.axvline(x=110, c='red')

            # #print len(ftx2),len(self.ftfeatures[i])

            # axarr.plot(ftx2, self.ftfeatures[i][0:-2]/np.max(self.ftfeatures[i][0:-2]), 'b--', label='FFT as stored', marker='.')
            axarr.set_xlim([40,120])
            axarr.set_ylim([0,1.2])

            axarr.legend(loc = 2, borderaxespad=0.)

        axarr.set_title('Normalized Power Spectrum')
        axarr.set_xlabel('Period [s]')
        axarr.set_ylabel('Normalized Power')
        return f

    def features_saver(self, outpath):
        """
        Save the features in the featureextracter.extractedlist to the file with the name outpath+filename from 
        featureextracter.corrlist as numpy array that then can be read in by the Neural Network.
        """
        if not os.path.exists(outpath): 
            os.makedirs(outpath)
        for i in range(len(self.extractedlist)): 
            np.save(outpath+self.corrlist[i], self.extractedlist[i])


def featureextracter_run(type='*.SAC', project = '/a20/', number_of_features=4, type2='all', velo=3800.):
    """
    Runs the whole featureextracter procedure for the Project and filetype with the number of features.
    """
    _,_,_,_, path_waveforms = info()
    _, outpath, subsets, classi = pathmaker(type='*.SAC', project = project)
    # path_raw,path_postprocessed,path_postprocessed_meta,_,_,_,outpath,_ = pathmaker(type='*.SAC', project = project)
    f = featureextracter(path_waveforms)
    f.normalize(type='positive_max')
    f.extractedlist = []
    f.controllist = []
    i = 0
    f.distance_attribute()
    l = len(f.stream_)
    it = 0
    print('Extracting FT features...')
    if type2 in['ft','all']:

        for trace in f.stream_:
            #print ('FT      Running trace %.0f of %.0f')%(i+1, len(f.stream_))
            _, representation2 = resampler(trace,  pmin=50, pmax=110, intervals = 18)
            #print len(representation2)
            f.extractedlist.append(representation2)
            i+=1
            it+=1
        f.features_saver(outpath+'ft/')


    f.extractedlist = []
    i = 0
    it = 0
    print('Extracting WBFE features...\nThis may take a while')
    if type2 in ['not ft', 'all']:
        for trace in f.stream_:
            trace.data = trace.data/np.max(trace.data)
            try: 
                extracted, representation = f.extract_feature(trace.data, 
                                                              distance = trace.stats.distance, 
                                                              number_of_features = number_of_features, velo=velo)
                f.extractedlist.append(representation)
            except: 
                #print 'ERROR ON TRACE: ', f.corrlist[i]
                f.extractedlist.append(np.zeros_like(representation))
            i+=1
            it+=1
        f.features_saver(outpath)


def featureextracter_load(type='all', project = '/a20/', number_of_features=2, period1 = '1', pertrue = False): 
    # #print glob.glob(outpath+'*.npy')
    """
    Loads the extracted features that were extracted before.
    """
    
    if type in ['good', 'bad']:
        path_raw,path_postprocessed,path_postprocessed_meta,path_synthetics,path_classifications_hand,path_classifications_synthetic,representationpath, path_subsets = pathmaker(type='*.SAC', project = project, pertrue=pertrue)

        f = featureextracter(path_postprocessed)
        f.features = []
        f.reconstructions = []
        f.ftfeatures = []
        classified = np.load(path_classifications_hand + 'classified_'+'hand_Pat2'+'.npy')


        if type == 'good': 
            x = [np.where(classified[:,1] == '1')]

        if type =='bad':
            x = [np.where(classified[:,1] == '0')]

        x = np.ravel(x).tolist()
        f.corrlist = np.array(f.corrlist)[[x]].tolist()
        stream = Stream(f.stream_[x[0]])
        t = 0
        for i in x: 
            if t>0:
                stream.append(f.stream_[i])
            t +=1
        f.stream_ = stream
        
    errors = 0

    if type == 'sub':
        path_raw,path_postprocessed,path_postprocessed_meta,path_synthetics,path_classifications_hand,path_classifications_synthetic,representationpath, path_subsets = pathmaker(type='', project = project)
        trainingsets = glob.glob(glob.glob(path_subsets+'/*')[0]+'/*')
        g = np.load(trainingsets[1])
        b = np.load(trainingsets[0])
        f = featureextracter(path_postprocessed, constraint=g)
        f.features = []
        f.reconstructions = []



    if type == 'all':
        path_postprocessed, representationpath, subsets, classi= pathmaker(type='*.SAC', project = project)
        f = featureextracter(path_postprocessed)
        f.features = []
        f.ftfeatures = []
        f.reconstructions = []
     
    errors = 0
    for stations in f.corrlist:
        #print stations
        tmp = np.load(representationpath+stations+'.npy')
        f.features.append(tmp)
        f.ftfeatures.append((np.load(representationpath+'/ft/'+stations+'.npy')))
        try: 
            f.reconstructions.append(f.reconstruct(tmp, number_of_features=number_of_features))
        except ValueError: 
            f.reconstructions.append(np.zeros_like(f.reconstructions[-1]))
            errors += 1
            #print 'error reconstructing', errors

    f.normalize(type='positive_max')
    f.distance_attribute()
    return f

# def plot_reconstructions2(self, f=None, axarr=None, c='black', foo=None, reconbackup=None):
#         """
#         Plot the reconstructions together with the power spectra (second column)
#         Returns a pyplot figure that then can be saved.
#         """
#         n = len(self.stream_)
#         # f, axarr = plt.subplots(n,3,sharex='col')
#         # f, axarr = plt.subplots(n,2,sharex='col', figsize = (15,15), dpi=600)
#         if f == None:
#             f, axarr = plt.subplots(1, sharex='col', figsize=(18, 6))
#             f.subplots_adjust(hspace=0.03)
#             f.subplots_adjust(wspace=0.03)
#             f.hold(True)
#         i = 0
#         for trace in self.stream_:
#             tr = trace.data
#
#             # ftx1, fty1, ftx2, fty2 = resampler(trace, pmin=50, pmax=110, intervals = 18, return_orig=True)
#
#             name = self.corrlist[i].split('.1.')[0]
#             distance = trace.stats.sac.dist
#
#             resid = tr - self.reconstructions[i]
#
#             if foo == None:
#                 # tr[tr == 0] = np.nan
#                 axarr.plot(tr, label='Correlation $\eta (t)$:  ' + name, c='dimgrey', linewidth=2.)
#
#             if reconbackup != None:
#                 self.reconstructions[i] = self.reconstructions[i] - reconbackup
#             # #print self.features[0][0]
#
#             # self.reconstructions[i][self.reconstructions[i]==0] = np.nan
#             if foo == None:
#                 # #print feats
#
#                 # #print self.features[0][i]
#                 # reconnan = np.copy(self.reconstructions[i])
#                 # reconnan[reconnan==0] = np.nan
#                 self.reconstructions[i][self.reconstructions[i] == 0] = np.nan
#                 axarr.plot(self.reconstructions[i], c=c, label=('WBFE Waveform $W_0$'), linewidth=2.)
#
#                 # #print len(self.features)
#                 # #print self.features[0][0], self.features[0][1]
#                 for ii in range(1):
#
#                     feats = self.features[0][ii]
#                     if feats[3] == 0:
#                         feats[3] = -1.
#                     axarr.scatter(feats[0], 0., s=200., color='black', facecolors='none', linewidth=2)
#
#                     axarr.scatter(feats[0] + feats[1], 0., s=200., color='black', facecolors='none', linewidth=2)
#
#                     axarr.scatter(feats[0] + feats[1] / 2, feats[-1] * feats[3], s=200., color='black', linewidth=2,
#                                   facecolors='none')
#
#                     axarr.annotate('eta', xy=(feats[0] + feats[1] / 2., feats[-1] * feats[3]), xycoords='data',
#                                    xytext=(-50, 20), textcoords='offset points',
#                                    arrowprops=dict(arrowstyle='->'), size=20)
#
#                     axarr.annotate('t1', xy=(feats[0], 0), xycoords='data',
#                                    xytext=(-20, -50), textcoords='offset points',
#                                    arrowprops=dict(arrowstyle='->', lw=2),
#                                    size=20)
#
#                     axarr.annotate('t2', xy=(feats[0] + feats[1], 0), xycoords='data',
#                                    xytext=(10, -50), textcoords='offset points',
#                                    arrowprops=dict(arrowstyle='->', lw=2),
#                                    size=20)
#
#                     axarr.fill_between(np.arange(len(tr)), tr, self.reconstructions[i], alpha=0.4, facecolor='grey')
#                     # axarr.fill_between(np.arange(len(tr)), tr-self.reconstructions[i], self.reconstructions[i], alpha=0.2, facecolor='grey')
#
#
#                     # axarr.axhline(0, linewidth =2, color='black')
#                     # axarr.axvline(x=feats[0], c=c)
#                     # axarr.annotate('$t_s$', xy=(feats[0],0.), xytext=(feats[0]-60,-1.), size='xx-large',  color=c)
#                     # axarr.axvline(x=feats[0]+feats[1], c=c)
#                     # axarr.annotate('$t_e$', xy=(feats[0]+feats[1],0.), xytext=(feats[0]+feats[1]+50,-1.), size='xx-large', color=c)
#                     # axarr.annotate('$t_d$', xy=(feats[0]+feats[1],0.), xytext=(feats[0]+feats[1]/2,-1.), size='xx-large', color=c)
#                     # axarr.annotate('$\eta(max)$', xy=(feats[0]+feats[1],0.), xytext=(feats[0]+feats[1]/2,1.05), size='xx-large', color=c)
#                     # axarr.annotate((' $\zeta =\t %.0f$ \n $F =\t %.2f$ \n $R =\t %.2f$ \n $t_l =\t %.0f$ \n $ p =\t %.0i $ \n $A_r =\t %.2f$')%(feats[4], feats[5], feats[6], feats[7],feats[3], feats[8]), xy=(feats[0]+feats[1],0.), xytext=(feats[0]+feats[1],0.3), size='x-large',  color=c)
#
#                     # axarr.annotate(str(feats[4]), xy=(1000,0.5), xytext=(1100,0.7), arrowprops=dict(facecolor='black',shrink=0.05))
#
#
#             else:
#                 feats = self.features[0][int(foo)]
#                 axarr.plot(self.reconstructions[i], c=c, label=('WBFE Waveform $W_%s$') % (str(int(foo))), linewidth=2.)
#                 axarr.axvline(x=feats[0], c=c)
#                 # axarr.annotate('$t_s$', xy=(feats[0],0.), xytext=(feats[0]-60,-1.), size='xx-large',  color=c)
#                 axarr.axvline(x=feats[0] + feats[1], c=c)
#                 # axarr.annotate('$t_e$', xy=(feats[0]+feats[1],0.), xytext=(feats[0]+feats[1]+50,-1.), size='xx-large', color=c)
#                 # axarr.annotate('$t_d$', xy=(feats[0]+feats[1],0.), xytext=(feats[0]+feats[1]/2,-1.), size='xx-large', color=c)
#                 # axarr.annotate('$\eta(max)$', xy=(feats[0]+feats[1],0.), xytext=(feats[0]+feats[1]/2,1.05), size='xx-large', color=c)
#                 if int(foo) % 2 != 0:
#                     axarr.annotate((
#                                    ' $\zeta =\t %.0f$ \n $F =\t %.2f$ \n $R =\t %.2f$ \n $t_l =\t %.0f$ \n $ p =\t %.0i $ \n $A_r =\t %.2f$') % (
#                                    feats[4], feats[5], feats[6], feats[7], feats[3], feats[8]),
#                                    xy=(feats[0] + feats[1], 0.), xytext=(feats[0], 0.3), size='x-large', color=c)
#                 else:
#                     axarr.annotate((
#                                    ' $\zeta =\t %.0f$ \n $F =\t %.2f$ \n $R =\t %.2f$ \n $t_l =\t %.0f$ \n $ p =\t %.0i $ \n $A_r =\t %.2f$') % (
#                                    feats[4], feats[5], feats[6], feats[7], feats[3], feats[8]),
#                                    xy=(feats[0] + feats[1], 0.), xytext=(feats[0] + feats[1], 0.3), size='x-large',
#                                    color=c)
#                     #
#             axarr.set_ylim([-1.2, 1.3])
#             axarr.set_xlim([0, len(tr)])
#
#             # if i!=len(self.stream_)-1:
#             # axarr[i,0].yaxis.set_major_formatter(plt.NullFormatter())
#             # axarr[i,0].xaxis.set_major_formatter(plt.NullFormatter())
#             axarr.legend(loc=4, borderaxespad=0., frameon=False)
#             # axarr.grid()
#
#             i += 1
#
#         axarr.set_title('Normalized Seismogram and Extracted Features (WBFE)')
#         # axarr[].set_title('Normalized Power spectrum')
#         # axarr[0,2].set_title('Residuals')
#
#         axarr.set_xlabel('Lag Time [s]')
#
#         axarr.set_ylabel('Normalized Amplitude')
#         # axarr.set_xlabel('Period [s]')
#         # axarr[-1,2].set_xlabel('Lag Time [s]')
#         return f, axarr
#         # plt.show()


if __name__=='__main__':
    raise ValueError('Please run elsewhere')
