from basic_functions import *

import numpy as np
import os, glob, sys

import matplotlib.pyplot as plt

from obspy import read, Trace


class postprocessing(ostream):
    """
    Postprocessing class on top of ostream to create the causal and acausal SAC correlation files 
    from the full correlations. 

    The pathmaker are defined in pathmaker.

    Takes the rawpath and puts the postprocessed traces into the traces path
    """
    def filter(self, fmin=0.01, fmax=0.02):
            """
            Filtering self.stream_ to the frequencies from fmin to fmax
            """ 
            self.stream_ = self.stream_.filter('bandpass', freqmin = fmin, freqmax=fmax, corners=3, zerophase=True)
            # self.stream_ = self.stream_.filter('bandpass', freqmin = fmin, freqmax=fmax, corners=0, zerophase=True)
            self.fmin = fmin
            self.fmax = fmax

    def velocity_windows(self, vmin=2000, vmax=6000):
        self.vmin = vmin
        self.vmax = vmax

    def causal(self): 
        """
        Get the causal time

        """
        for trace in self.stream_: 
            trace.data = trace.data[self.time>=0]
        self.causaltime = self.time[self.time>=0]

    def acausal(self): 
        """
        Get the acausal time
        """
        self.stream_2 = self.stream_.copy()
        for trace in self.stream_2:
            trace.data = np.flipud(trace.data[self.time<=0])
            self.acausaltime = self.time[self.time<0]

    def timewindow(self, maxlen=2700): 
        """ 
        Window the trace.data in self.stream_ to a maximum length of maxlen
        """
        for trace in self.stream_:
            trace.data = trace.data[self.causaltime<=maxlen]
        try: 
            for trace in self.stream_2: 
                trace.data = trace.data[self.causaltime<=maxlen]
        except: 
            print 'ERROR WITH ACAUSAL PART, PLEASE CHECK !'
        self.causaltime = self.causaltime[self.causaltime<=maxlen]


    def write_stream(self, target_location=None, 
                     nameid='.1', 
                     format_='SAC', autocorrs = False): 

        """
        Write the stream to the location in filename
        """

        self.format_out = format_
        print len(self.stream_[0].data)
        for trace in self.stream_:
            if autocorrs == False:
                if trace.stats.sac.dist > 0:
                    label = self.get_info_from_sac(trace)
                    filename = target_location+label+nameid+'.'+format_
                    trace.write(filename, format=format_)

    def write_stream_acausal(self, target_location=None, 
                     nameid='.1', 
                     format_='SAC', autocorrs = False): 
        """
        Write the acausal parts

        """
        self.format_out = format_
        print len(self.stream_2[0].data)
        for trace in self.stream_2:
            if autocorrs == False:
                if trace.stats.sac.dist > 0:
                    label = self.get_info_from_sac_acausal(trace)
                    filename = target_location+label+nameid+'.'+format_
                    trace.write(filename, format=format_)

    def metafile(self, meta_file_loc = '/home/patrick/1Thesis/waveforms_classification/waveforms/a11_meta_postproc.txt'): 
        """ Save a Metafile  for the postprocessing"""

        with open(meta_file_loc, 'a') as f: 
            f.write(('Maximum Distance           \t = %s km \n')%(self.maxdist))
            f.write(('Minimum Frequency (Period) \t = %s Hz (%s s)\n')%(self.fmin, (1/self.fmin)))
            f.write(('Maximum Frequency (Period) \t = %s Hz (%s s)\n')%(self.fmax, (1/self.fmax)))
            # f.write(('Sampling_rate              \t = %s s\n')%(d.sampling_rate))
            f.write(('Part of the waveform:      \t %s \n')%('causal and acausal'))
            f.write(('Output Format:             \t %s \n')%(self.format_out))
            f.write(('Number of Distance classes \t = %s \n')%(self.n))
            f.write(('Number of Files            \t = %s \n')%(len(self.stream_)))


def postprocess_run(type='*.SAC', project = '/a11/', fmin=0.01, fmax=0.02, maxlen =2700): 
    """
    Run the whole postprocessing procedure
    """

    path_raw,path_postprocessed,path_postprocessed_meta,_,_,_,_,_ = pathmaker(type='*.SAC', project = project, postproc=True)
    print ('Postprocessing initialized with parameters:\nPeriod band from %.1f to %.1f seconds') % (1/fmax, 1/fmin)
    print path_raw, path_postprocessed
    p = postprocessing(path_raw)
    p.distance_attribute()
    # print p.corrlist[5]
    print 'Getting Time....'
    p.get_time()
    print 'Filtering...'
    p.filter(fmin=fmin, fmax=fmax)
    print 'FREQUENCY BANDS:', p.fmin, p.fmax
    p.get_max_dist()
    p.velocity_windows()
    print 'Acausal...'
    p.acausal()
    print 'Causal....'
    p.causal()
    # p.acausal()
    print ('saving Stream with maximum lag time of %.1f') % (maxlen)
    p.timewindow(maxlen=maxlen)

    p.write_stream(target_location=path_postprocessed)
    p.write_stream_acausal(target_location=path_postprocessed)
    # p.metafile(meta_file_loc=path_postprocessed+'_postproc_meta.txt')


    # p.plot_stream()

if __name__ == '__main__':
    print 'PLEASE CALL FROM POSTPROC FEATURES SHELL'
    # postprocess_run( project = '/glob/', fmin=0.01, fmax=0.02, maxlen=2700)
